# Install-script for WikiArchive extensions for JSPWiki.
# Jens Gulden (jgulden@cs.tu-berlin.de)

##############################################################################
# EDIT HERE
#
# These paths may need to be adopted to your configuration.

# run a test with a freshly unpacked JSPWiki distribution
# (instead of installing to an existing JSPWiki)
TEST="yes"

# Java installation directory
JAVA=/usr/java/java

# Tomcat installation directory
TOMCAT=/usr/share/tomcat4

# Temp-dir for unpacking archives
TMP=/tmp

# allow blanks in filenames (i.e. only LF as list-separator)
IFS="
"

# this is where you unzipped the wikiarchive.zip
# (directory "wikiarchive/" is part of the zip file,
# so unzip in /tmp to get the default)
INSTALLFROM=${TMP}/wikiarchive

# set to your configuration, e.g. tomcat-webapps dir /usr/share/tomcat/webapps
WEBAPPS=${TMP}/webapps

# Basic JSPWiki installation directory
INST=${WEBAPPS}/JSPWiki

# WEB-INF folder
WEBINF=${INST}/WEB-INF

# templates folder (patches may run with other templates, but tested for default templates only)
TEMPLATES=${INST}/templates/default

# JSPWiki source files
#JSPWIKI_SRC=${INST}/src

# installed WikiArchive source files
SRC=${WEBINF}/src

# additional JSPWiki libraries / .jar-files
LIB=${WEBINF}/lib

# target directory for compiled classes
BUILD=${WEBINF}/classes

# original JSPWikibinary distribution for testing
JSPWIKI_DISTRIBUTION=/d/downloads/jspwiki/JSPWiki-2.4.102-bin.zip

##############################################################################

# for testing

if [[ "$TEST" == "yes" ]]; then
  echo "TEST: installing initial JSPWiki v 2.4 to ${WEBAPPS}"
  
  cd ${TMP}
  rm -r JSPWiki/
  unzip -q /d/downloads/jspwiki/JSPWiki-2.4.102-bin.zip
  
  mkdir ${WEBAPPS}
  rm -r ${INST}
  mkdir ${INST}
  cd ${INST}
  unzip ${TMP}/JSPWiki/JSPWiki.war
fi

# end of testing

##############################################################################

echo "Installing WikiArchive extensions from ${INSTALLFROM} to ${INST}..."

# --- apply diffs to JSPs ---

echo "Applying diffs to templates JSPs."

cd ${INSTALLFROM}/templates
cp -r * ${TEMPLATES}
cd ${TEMPLATES}
for i in `ls *.diff`; do
  echo $i
  patch ${i/".diff"} $i
done
cd editors
patch plain.jsp plain.jsp.diff

# --- apply diffs to config ---

echo "Applying diffs to configuration."

cd ${INSTALLFROM}/WEB-INF
cp -r * ${WEBINF}
cd ${WEBINF}
for i in `ls *.diff`; do
  echo $i
  patch ${i/".diff"} $i
done

# --- compile java sources ---

echo "Compiling sources..."

mkdir ${SRC}
mkdir ${BUILD}

cd ${INSTALLFROM}

cp -r src ${SRC}/..

LIBS=""
cd ${TOMCAT}/common/lib
for i in `ls *.jar`; do
  LIBS="${LIBS}${TOMCAT}/common/lib/${i}:"
done
cd ${LIB}
for i in `ls *.jar`; do
  LIBS="${LIBS}${LIB}/${i}:"
done

echo "additional classpath ${LIBS}"

cd ${SRC}

JAVAFILES="org/devlib/schmidt/imageinfo/ImageInfo.java "
for i in `find -name *.java | grep ./de/`; do
  JAVAFILES="${JAVAFILES}${i} "
done

echo "${JAVAFILES}" > ${TMP}/sources.list

echo "compiling..."

cd ${SRC}
echo "${JAVA}/bin/javac -d ${BUILD} -cp ${LIBS} @${TMP}/sources.list"
${JAVA}/bin/javac -d ${BUILD} -cp ${LIBS} @${TMP}/sources.list

if [[ "$TEST" == "yes" ]]; then
  # copy example wiki pages
  cp -r ${INSTALLFROM}/wikipages ${WEBINF}
  
  # fix page-dir from original jspwiki.properties so we can start up
  W=${WEBINF//\//\\\/} # replace all "/" with "\/"
  sed "s/\/p\/web\/www-data\/jspwiki\//${W}\/wikipages/g" ${WEBINF}/jspwiki.properties > ${TMP}/jspwiki.properties && mv ${TMP}/jspwiki.properties ${WEBINF}
  count=1
  for i in gif png jpg jpeg GIF PNG JPG JPEG; do
    echo "jspwiki.translatorReader.inlinePattern.${count}=*.${i}" >> ${WEBINF}/jspwiki.properties
    count=$(($count+1))
  done
  echo "jspwiki.security = off" >> ${WEBINF}/jspwiki.properties
  
  chmod -R a+rw ${WEBAPPS} # full access (so server-user tomcat4 can change files, it's just a test anyway)
  echo "TEST: Installation has been created at ${WEBAPPS}. Symlink this as your "webapps" dir and restart the server."
  echo "      Then point your browser to http://localhost:8180/JSPWiki/Wiki.jsp."
fi
