/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.plugin;

import com.ecyrd.jspwiki.*;
import com.ecyrd.jspwiki.attachment.*;
import com.ecyrd.jspwiki.plugin.*;
import com.ecyrd.jspwiki.providers.*;
import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import de.tuberlin.cs.flp.jspwiki.plugin.Picture;
import de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive.ArchiveEntry;
import de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive.ArchiveHandlerWiki;
import de.tuberlin.cs.flp.jspwiki.tags.FileIconTag;
import de.tuberlin.cs.flp.jspwiki.util.*;
import de.tuberlin.cs.flp.jspwiki.util.WikiToolbox;
import java.io.*;
import java.util.*;

/**
 * <strong>[{Category <br/>
 * [page=&quot;..&quot;] <br/>
 * format=&quot;_list_|gallery&quot; <br/>
 * sort=&quot;_alphabetic_|none&quot; <br/>
 * headlines=&quot;_alphabetic_lnone|&quot; <br/>
 * attachments=&quot;yes|_no_&quot; <br/>
 * gallery-width=n (default: 4) <br/>
 * thumbnails=&quot;yes|_no_&quot;<br/>
 * thumbnailWidth=n<br/>
 * thumbnailHeight=n<br/>
 * pagesize=n (maximum number of entries to be displayed per page, default: no limit) }]</strong>
 * <p>Lists all pages that are associated with the current page as their Category. The output format can be configured in several ways according to the parameters. Special (unlike ReferringPagesPlugin): Only those pages are considered members of the category which contain a link where the category name is seperated from the prefix &quot;Category&quot; WITH A BLANK. This way it becomes possible to link to the category page without declaring the referring page a member of the category, by NOT using a blank between &quot;Category&quot; and the category name. Examples: using &quot;[Category Glossary]&quot; DOES mark a page as a member of the category, using [CategoryGlossary] simply is a link to the category page without declaring the page in which it appears as a member of the category. Note that &quot;Category&quot; is not necessarily word that marks a page a category page, simply the first uppercase part of the page's name is used.</p>
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e4c]
 * @author Jens Gulden
 */
public class Category implements WikiPlugin {

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7102]
 */
    public static final String PARAM_PAGE = "page";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70fc]
 */
    public static final String PARAM_FORMAT = "format";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70f6]
 */
    public static final String VALUE_FORMAT_GALLERY = "gallery";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70f0]
 */
    public static final int FORMAT_LIST = 0;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70ea]
 */
    public static final int FORMAT_GALLERY = 1;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70e4]
 */
    public static final String PARAM_SORT = "sort";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70de]
 */
    public static final int SORT_NONE = -1;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70d8]
 */
    public static final int SORT_ALPHABETIC = 0;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70d2]
 */
    public static final String VALUE_SORT_ALPHABETIC = "alphabetic";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70cc]
 */
    public static final String PARAM_HEADLINES = "headlines";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70c6]
 */
    public static final int HEADLINES_NONE = -1;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70c0]
 */
    public static final int HEADLINES_ALPHABETIC = 0;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70ba]
 */
    public static final String VALUE_HEADLINES_ALPHABETIC = "alphabetic";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70b4]
 */
    public static final String PARAM_ATTACHMENTS = "attachments";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70ae]
 */
    public static final String PARAM_THUMBNAILS = "thumbnails";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70a8]
 */
    public static final String PARAM_THUMBNAIL_SIZE = "thumbnail-size";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm70a2]
 */
    public static final String DEFAULT_THUMBNAIL_SIZE = "thumbnail";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm709c]
 */
    public static final String PARAM_GALLERY_COLUMNS = "gallery-columns";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7096]
 */
    public static final int DEFAULT_GALLERY_COLUMNS = 4;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7090]
 */
    public static final String PARAM_PAGESIZE = "pagesize";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm708a]
 */
    public static final int DEFAULT_PAGESIZE = -1;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7084]
 */
    public static final String HTTP_PARAM_PAGE = "pagenr";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm707e]
 */
    public static final int DEFAULT_PAGE = 1;

/**
 * Executes the plugin.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7075]
 * @return HTML to be inserted into page
 * @param context The WikiContext.
 * @param params Parameters specified at plugin inclusion.
 * @throws PluginException if an error occurs
 */
    public String execute(WikiContext context, Map params) throws PluginException {        
        WikiEngine engine = context.getEngine();
        CategoryManager cm = CategoryManager.getInstance(engine); // own extension, static access
        AttachmentManager am = engine.getAttachmentManager();
        
        // get parameters
        String pagename = context.getPage().getName();
        String category = (String)params.get(PARAM_PAGE);
        if (category == null) { // usual case: use current page as category page
            category = pagename;
        }
        String p = (String)params.get(PARAM_SORT);
        int sortMode = ( (p != null) && (p.equalsIgnoreCase(VALUE_SORT_ALPHABETIC)) ) ? SORT_ALPHABETIC : SORT_NONE;
        p = (String)params.get(PARAM_FORMAT);
        int formatMode = ( (p != null) && (p.equalsIgnoreCase(VALUE_FORMAT_GALLERY)) ) ? FORMAT_GALLERY : FORMAT_LIST;
        p = (String)params.get(PARAM_HEADLINES);
        int headlinesMode = ( (p != null) && (p.equalsIgnoreCase(VALUE_HEADLINES_ALPHABETIC)) ) ? HEADLINES_ALPHABETIC : HEADLINES_NONE;//Toolbox.findIndex((String)params.get(PARAM_HEADLINES), new String[] {VALUE_HEADLINES_ALPHABETIC}, SORT_ALPHABETIC);
        p = (String)params.get(PARAM_ATTACHMENTS);
        boolean showAttachments = (p==null)||(! (p.equalsIgnoreCase("no") || p.equalsIgnoreCase("false")) );
        p = (String)params.get(PARAM_THUMBNAILS);
        boolean showThumbnails = (p!=null) && ( p.equalsIgnoreCase("yes") || p.equalsIgnoreCase("on") || p.equalsIgnoreCase("true") );
        p = (String)params.get(PARAM_GALLERY_COLUMNS);
        int galleryColumns = DEFAULT_GALLERY_COLUMNS;
        if ( p != null) {
            try {
                galleryColumns = Integer.parseInt( p );
            } catch (NumberFormatException nfe) {
                // nop
            }
        }
        String thumbnailSize = (String)params.get(PARAM_THUMBNAIL_SIZE);
        if ( thumbnailSize == null ) {
            thumbnailSize = DEFAULT_THUMBNAIL_SIZE;
        }
        p = (String)params.get(PARAM_PAGESIZE);
        int pagesize = DEFAULT_PAGESIZE;
        if ( p != null) {
            try {
                pagesize = Integer.parseInt( p );
            } catch (NumberFormatException nfe) {
                // nop
            }
        }
        p = context.getHttpParameter(HTTP_PARAM_PAGE);
        int page = DEFAULT_PAGE;
        if ( p != null) {
            try {
                page = Integer.parseInt( p );
            } catch (NumberFormatException nfe) {
                // nop
            }
        }
        
        // get all pages linking to this one  // NEIN:, but WITH A BLANK between "Category" (or equiv.) and the actual category name
        Collection pages = cm.getExistingCategoryMembers(category);
        Collection toBeCreated = cm.getNewCategoryMembers(category);        
        
        if (sortMode == SORT_ALPHABETIC) {
            List pagesAlphabetic = new ArrayList(pages);
            Collections.sort(pagesAlphabetic);
            pages = pagesAlphabetic;
        }
        
        // page-navigator, if necessary
        String pageNavi;
        if ( ( pagesize > 0 ) && ( pages.size() > pagesize ) ) {
            int lastpage = ( pages.size() / pagesize ) + 1;
            StringBuffer html = new StringBuffer();
            String url = engine.getViewURL(pagename);
            url += ( ( url.indexOf('?') == -1 ) ? '?' : '&' ) + HTTP_PARAM_PAGE + '=';
            html.append("<table align='center' class='categoryPagenavi'><tr>");
            html.append("<td class='categoryPagenavi'>");
            if ( page > 1 ) {
                html.append("<a class='categoryPagenavi' href='" + url + ( page - 1 ) +"'>");
            } else {
                //html.append("<i>");
            }
            html.append("&lt;&lt;");
            if ( page > 1 ) {
                html.append("</a>");
            } else {
                //html.append("</i>");
            }
            html.append("</td>");
            
            for (int i = 1; i <= lastpage; i++ ) {
                html.append("<td class='categoryPagenavi'>");
                if ( page != i ) {
                    html.append("<a class='categoryPagenavi' href='" + url + i +"'>");
                } else {
                    html.append("<b>");
                }
                html.append(String.valueOf(i));
                if ( page != i ) {
                    html.append("</a>");
                } else {
                    html.append("</b>");
                }
                html.append("</td>");
            }
            html.append("<td class='categoryPagenavi'>");
            if ( page < lastpage ) {
                html.append("<a class='categoryPagenavi' href='" + url + ( page + 1 ) +"'>");
            } else {
                //html.append("<i>");
            }
            html.append("&gt;&gt;");
            if ( page < lastpage ) {
                html.append("</a>");
            } else {
                //html.append("</i>");
            }
            html.append("</td>");
            
            html.append("</tr></table>");
            pageNavi = html.toString();
        } else {
            pageNavi = "";
        }
        // create wiki markup
        StringBuffer html = new StringBuffer();
        StringBuffer wikitext = new StringBuffer();
        
        if ( ! toBeCreated.isEmpty() ) {
            StringBuffer newentries = new StringBuffer();
            for (Iterator it = toBeCreated.iterator(); it.hasNext(); ) {
                String name = (String)it.next();
                newentries.append("["+name+"]\\\\\n");
            }
            wikitext.append("\n\n! New\n\n");
            wikitext.append(newentries);
            wikitext.append("\n----\n");
            String h = engine.textToHTML(context, wikitext.toString());
            html.append(h);
        }
        
        html.append("<a name='topofpage'/>\n");
        
        StringBuffer headlineLetters = new StringBuffer();
        char currentHeading = (char)0;
		for (Iterator it = pages.iterator(); it.hasNext();) {
			String name = (String) it.next();
			char firstCharUpperCase = Character.toUpperCase(name.charAt(0));
			// letter-heading
			if (currentHeading != firstCharUpperCase) {
				currentHeading = firstCharUpperCase;
				headlineLetters.append(currentHeading);
			}
		}
        StringBuffer head = new StringBuffer();
        if ( headlinesMode == HEADLINES_ALPHABETIC ) {
            // alphabet-navigator at the top
            String hl = headlineLetters.toString();
            head.append("<center><table class='categoryAlphabetMenu' padding='20'><tr>");
            for (char c = 'A'; c <= 'Z'; c++) {
                if (hl.indexOf(c) != -1) {
                    head.append("<td class='categoryAlphabetMenu'><a class='categoryAlphabetMenu' href='#"+c+"'>"+c+"</a></td>");
                } else {
                    head.append("<td class='categoryAlphabetMenuUnused'>"+c+"</td>");
                }
            }
            head.append("</tr></table></center>\n");
        }
        html.append(head);
                
        int startentry = pagesize * ( page - 1 );
        int endentry = (pagesize * page) - 1;
        currentHeading = (char)0;
        int columnCount = 0; // 0 .. galleryColumns-1
        int columnWidthPercent = Math.round( 100 / galleryColumns );
        String css = showThumbnails ? "categoryGalleryThumbnail" : "categoryGallery";
        Picture pluginPicture = null;
        Map pluginPictureParams = null;
        if (formatMode == FORMAT_GALLERY) {
            html.append("<table width='100%' class='categoryGallery'>\n");
        }
        int count = 0;
        for (Iterator it = pages.iterator(); it.hasNext(); ) {
            boolean display = ( pagesize <= 0 ) || ( ( count >= startentry ) && ( count <= endentry ) );
            wikitext = new StringBuffer();
            String name = (String)it.next();
            if (headlinesMode == HEADLINES_ALPHABETIC) {
                char firstCharUpperCase = Character.toUpperCase(name.charAt(0));
                // letter-heading
                if (currentHeading != firstCharUpperCase) {
                    currentHeading = firstCharUpperCase;
                    
                    if ( display ) {
                        headlineLetters.append(currentHeading);
                    }
                    
                    //wikitext.append("\n!"+currentHeading+"\n\n");
                    if (formatMode == FORMAT_GALLERY) {
                        if ( columnCount != 0 ) { // need to end current row first?
                            html.append("<td colspan='"+(galleryColumns - columnCount)+"' class='categoryGalleryAlphabetRemainder'/></tr>");
                            columnCount = 0;
                        }
                        if ( display ) {
                            html.append("<tr><td colspan='"+galleryColumns+"' class='categoryGalleryAlphabet'>");
                        }
                    }
                    if ( display ) {
                        html.append("<a name='"+currentHeading+"'/><table class='toTop'><tr><td class='toTopSpace'/><td class='toTop'><a href='#topofpage'>top</a></td></tr></table>");
                        html.append( heading(String.valueOf(currentHeading)) );
                        if (formatMode == FORMAT_GALLERY) {
                            html.append("</td></tr>");
                        }
                    }
                }
            }
            if ( display ) {
                Collection attachments = null;
                StringBuffer att = new StringBuffer();
                WikiPage wikipage = engine.getPage(name);
                if ( showAttachments || showThumbnails ) {
                    // attachment available?
                    if (wikipage != null) {
                        try {
                            attachments = am.listAttachments(wikipage);
                            if ( showAttachments ) {
                                if (attachments.size() > 0) {
                                    att.append(" ");
                                }
                                for (Iterator at = attachments.iterator(); at.hasNext(); ) {
                                    Attachment a = (Attachment)at.next();
                                    String filename = a.getFileName();
                                    if (filename != null) {
                                        //if ( (! showThumbnails) || ( ! cm.isImage(filename) ) ) { // avoid link to image as attachment if used for thumbnails
                                        if ( ! WikiToolbox.isImage(engine, filename) ) { // avoid link to image as attachment
                                            String iconLink = FileIconTag.getFileIcon(engine, filename);
                                            String attachmentLink = name+"/"+filename;
                                            String suffix;
                                            int pos = filename.lastIndexOf('.');
                                            if (pos-1 < filename.length()) {
                                                suffix = filename.substring(pos+1);
                                            } else {
                                                suffix = "...";
                                            }
                                            att.append(" ["+iconLink+"]["+suffix+"|"+attachmentLink+"]");
                                        }
                                    }
                                }
                            }
                        } catch (ProviderException pe) {
                            // nop, ignore
                        }
                    }
                }
                // output entry
                if ( formatMode == FORMAT_GALLERY ) {
                    if ( columnCount == 0 ) {
                        html.append("<tr>");
                    }
                    html.append("<td width='"+columnWidthPercent+"%' class='"+css+"'>");
                }
                if ( showThumbnails ) {
                    if ( attachments != null ) {
                        // call plugin Picture
                        String url = engine.getViewURL(name);
                        html.append( "<a href='"+url+"'>" + Picture.imageHTML(context, name, "thumbnail", css, "", false, false) + "</a><br/>" );
                    }
                }
                
                // try to derive a more detailed title from the page
                String fulltitle = deriveFullTitle(name, wikipage, engine);
                if (fulltitle != null) {
                	fulltitle = " ''" + fulltitle + "'' ";
                } else {
                	fulltitle = "";
                }
                
                wikitext.append("[[ ["+name+"] ]"+ fulltitle + att.toString());//+"\\\\\n");
                String h = engine.textToHTML(context, wikitext.toString());
                html.append(h);
                if ( formatMode == FORMAT_GALLERY ) {
                    html.append("</td>\n");
                    if ( columnCount == galleryColumns-1 ) {
                        html.append("</tr>\n");
                    }
                } else {
                    html.append("<br/>\n");
                }
            }
            
            columnCount++;
            if ( columnCount == galleryColumns ) {
                columnCount = 0;
            }
            
            count++;
        }
        
        if ( formatMode == FORMAT_GALLERY ) {
            html.append("</table>\n");
        }
        
        if ( formatMode == FORMAT_GALLERY ) {
            html.append("<center>"); // workaround for bad stylesheet interpretation
            html.append(pageNavi);
            html.append("</center>");
        } else {
            html.append(pageNavi);
        }
        return html.toString();
    } 

/**
 * 
 * 
 * @poseidon-object-id [Ie4883fm112767c1b91mm35f6]
 * @return 
 * @param name 
 * @param wikipage 
 * @param engine 
 */
    protected String deriveFullTitle(String name, WikiPage wikipage, WikiEngine engine) {        
        return null; // implemeted by subclass WikiArchive
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm706c]
 * @return 
 * @param category 
 */
    public static String categoryPageLink(String category) {        
        List cc = WikiToolbox.splitCamelCase(category);
        String link;
        if (cc.size() > 1) {
            link = "[" + (String)cc.get(0) + " ";
            for (int i = 1; i < cc.size(); i++) {
                link += (String)cc.get(i);
            }
            link += "]";
        } else {
            link = "[" + category + "]";
        }
        return link;
    } 

/**
 * Formats a string as HTML-heading.
 * 
 * @poseidon-object-id [Ie4883fm112767c1b91mm422d]
 * @return The formatted HTML string.
 * @param s The string to be formatted.
 */
    protected static String heading(String s) {        
            return "<h4 class='categoryMenu'><font color='black'>"+s+"</font></h4>"; // (font-tag is workaround for bad stylesheet interpretation, avoid blue-colored headings)
    } 
 }
