/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.plugin;

import com.ecyrd.jspwiki.*;
import com.ecyrd.jspwiki.attachment.*;
import com.ecyrd.jspwiki.plugin.*;
import com.ecyrd.jspwiki.providers.*;
import com.ecyrd.jspwiki.util.HttpUtil;
import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive.*;
import de.tuberlin.cs.flp.jspwiki.tags.FileIconTag;
import de.tuberlin.cs.flp.jspwiki.util.*;
import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import java.util.zip.*;
import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspWriter;

/**
 * <strong>[{WikiArchive <br/>
 * (all parameters from the 'Category'-plugin, plus:) <br/>
 * [import=type1,type2,...] <br/>
 * [export=type1,type2,...] <br/>
 * [io=type1,type2,...] (sets both import and export, usual usage) <br/>
 * [categoryMarker='text with link to category-page'] (page-footer for import/exportm marking the page as a member of its category) <br/>
 * [usePageLinks] (use outgoing links from page, instead of category member. Useful e.g. for exporting from legacy-pseudo-categories and later create a new category) <br/>
 * [zipAttachments] (enable zip file import/export which may include attachments) }]</strong>
 * <p>See also configuration parameters: ...enableDelete ...enableZipAttachments</p>
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e46]
 * @author Jens Gulden
 */
public class WikiArchive extends Category {

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72f8]
 */
    public static final String PARAM_IMPORT = "import";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72f2]
 */
    public static final String PARAM_EXPORT = "export";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72ec]
 */
    public static final String PARAM_IO = "io";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72e6]
 */
    public static final String PARAM_CATEGORY_MARKER = "categoryMarker";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72e0]
 */
    public static final String PARAM_USE_OUTGOING_LINKS = "usePageLinks";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72da]
 */
    public static final String PARAM_ZIP_ATTACHMENTS = "zipAttachments";

/**
 * 
 * 
 * @poseidon-object-id [Ie4883fm112767c1b91mm4288]
 */
    public static final String PARAM_MENU_POSITION = "menuPosition";

/**
 * Constant MODE_NONE.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72d4]
 */
    public static final int MODE_NONE = 0;

/**
 * Constant MODE_UPDATE.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72ce]
 */
    public static final int MODE_UPDATE = 1;

/**
 * Constant MODE_INSERT.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72c8]
 */
    public static final int MODE_INSERT = 2;

/**
 * Constant MODE_BOTH.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72c2]
 */
    public static final int MODE_BOTH = MODE_UPDATE | MODE_INSERT;

/**
 * Constant MODE_SINGLE.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72bc]
 */
    public static final int MODE_SINGLE = 4;

/**
 * Constant EXPORT.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72b6]
 */
    public static final int EXPORT = 1;

/**
 * Constant IMPORT.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72b0]
 */
    public static final int IMPORT = 2;

/**
 * Constant BOTH.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72aa]
 */
    public static final int BOTH = IMPORT | EXPORT;

/**
 * Constant ATTRIBUTE_EXPORT_RESULT.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm72a4]
 */
    public static final String ATTRIBUTE_EXPORT_RESULT = "wikiarchive.export.data";

/**
 * Constant ATTRIBUTE_EXPORT_CONTENTTYPE.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm729e]
 */
    public static final String ATTRIBUTE_EXPORT_CONTENTTYPE = "wikiarchive.export.contenttype";

/**
 * Constant ATTRIBUTE_EXPORT_CONTENTDISPOSITION.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7298]
 */
    public static final String ATTRIBUTE_EXPORT_CONTENTDISPOSITION = "wikiarchive.export.contentdisposition";

/**
 * Character encoding used by uploaded files and used for generating download data.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7292]
 */
    public static final String CHARACTER_ENCODING = "ISO-8859-1";

/**
 * Temporary directory used if not otherwise configured via
 * parameter "jspwiki.plugin_name.uploadTempDir" in WEB-INF/jspwiki.properties.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm728c]
 */
    public static final String DEFAULT_TEMP_DIR = "/tmp";

/**
 * Executes the plugin.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7283]
 * @return HTML to be inserted into page
 * @param context The WikiContext.
 * @param params Parameters specified at plugin inclusion.
 * @throws PluginException if an error occurs
 */
    public String execute(WikiContext context, Map params) throws PluginException {        
        // get environment
        WikiEngine engine = context.getEngine();
        CategoryManager cm = CategoryManager.getInstance(engine);
        String thisPage = context.getPage().getName();
        String pagename = (String)params.get(PARAM_PAGE);
        if (pagename == null) { // usual case: use current page as category page
            pagename = thisPage;
        }
        HttpServletRequest request = context.getHttpRequest();
        
        // get parameters
        Collection importFormats = WikiToolbox.split( (String)params.get( PARAM_IMPORT ) );
        Collection exportFormats = WikiToolbox.split( (String)params.get( PARAM_EXPORT ) );
        Collection ioFormats = WikiToolbox.split( (String)params.get( PARAM_IO ) );
        Map ioHandlers = new HashMap();
        for ( Iterator it = ioFormats.iterator(); it.hasNext(); ) { // make sure all in ioFormats also available in both importFormats and exportFormats
            String importexport = (String)it.next();
            if ( ! importFormats.contains(importexport) ) {
                importFormats.add(importexport);
            }
            if ( ! exportFormats.contains(importexport) ) {
                exportFormats.add(importexport);
            }
        }
        for ( Iterator it = importFormats.iterator(); it.hasNext(); ) { // on the other hand, make sure all in importFormats and exportFormats are also available in ioFormats
            String f = (String)it.next();
            if ( ! ioFormats.contains(f) ) {
                ioFormats.add(f);
            }
        }
        for ( Iterator it = exportFormats.iterator(); it.hasNext(); ) { // on the other hand, make sure all in importFormats and exportFormats are also available in ioFormats
            String f = (String)it.next();
            if ( ! ioFormats.contains(f) ) {
                ioFormats.add(f);
            }
        }
        String categoryMarker = (String)params.get(PARAM_CATEGORY_MARKER);
        if ( categoryMarker == null ) {
            categoryMarker = cm.getDefaultCategoryMemberPhrase() + " [" + pagename + "].";
        }
        boolean useOutgoingLinks = params.containsKey( PARAM_USE_OUTGOING_LINKS );
        boolean enableDelete = Boolean.valueOf(engine.getWikiProperties().getProperty(getPropertyName("enableDelete"), "false")).booleanValue();
        boolean enableZipAttachments = params.containsKey( PARAM_ZIP_ATTACHMENTS );
        String menuPositionStr = (String)params.get( PARAM_MENU_POSITION );
        boolean menuAtTop = (menuPositionStr != null) && (menuPositionStr.equalsIgnoreCase("top"));
        try {
            // initialize import and export handlers
            Map formatLabels = new HashMap();
            for ( Iterator it = ioFormats.iterator(); it.hasNext(); ) {
                String format = (String)it.next();
                String formatPropertyPrefix = getPropertyName( "handler." + format );
                String className = engine.getWikiProperties().getProperty( formatPropertyPrefix );
                if ( className != null ) { // silently ignore if handler unknown
                    // get ArchiveHandler instance
                    Class handlerClass = Class.forName( className );
                    ArchiveHandler handler = (ArchiveHandler)handlerClass.newInstance();
                    Map handlerConfig = WikiToolbox.subMap( engine.getWikiProperties(), formatPropertyPrefix );
                    handler.init( engine, handlerConfig );
                    // get format label
                    String label = engine.getWikiProperties().getProperty(formatPropertyPrefix+".label");
                    if ( label == null ) {
                        label = WikiToolbox.capitalize(format);
                    }
                    formatLabels.put(format, label);
                    ioHandlers.put(format, handler);
                }
            }
            ArchiveHandler wikiIOHandler = new ArchiveHandlerWiki(categoryMarker);
            String importMessages = null;
            String exportMessages = null;
            String createMessages = null;
            boolean skipGUI = false;
            // variables to hold http-parameters:
            String exportParam = null;
            String createParam = null;
            String deleteParam = null;
            String importParam = null;
            String importFilename = null;
            String importFilePath = null;
            String importTempfile = null;
            int mode = MODE_BOTH | MODE_SINGLE;
            boolean importzip = false;
            boolean exportzip = false;
            Collection single = null;
            String singleFormat;
            StringBuffer html = new StringBuffer();
            // --- Import ---
            if (MultipartFormData.isMultipartFormData(request)) {
                Map parameters = MultipartFormData.getMultipartFormData(request, CHARACTER_ENCODING);
                importParam = (String)parameters.get("import");
                importzip = parameters.containsKey("importzip");
                if (importParam != null) {
                    String si = (String)parameters.get("importmode");
                    if (si != null) {
                        mode = Integer.parseInt(si);
                    } else {
                        //nop mode = MODE_BOTH | MODE_SINGLE;
                    }
                    importFilename = (String)parameters.get("importfile_filename");
                    importFilePath = (String)parameters.get("importfilepath");
                    importTempfile = (String)parameters.get("importtempfile");
                    String uploadString = null;
                    String tempdirName = engine.getWikiProperties().getProperty(getPropertyName("uploadTempDir"), DEFAULT_TEMP_DIR);
                    File tempdir = new File(tempdirName);
                    if (importFilename!=null && importFilename.length()>0) { // file has just been uploaded
                        uploadString = (String)parameters.get("importfile");
                        // buffer in temp-dir for later recall
                        File f = File.createTempFile("wikiarchive", ".tmp", tempdir);
                        FileOutputStream out = new FileOutputStream(f);
                        byte[] b = uploadString.getBytes(CHARACTER_ENCODING); // ?need decoding of binary data here?
                        out.write(b);
                        out.close();
                        importTempfile = f.getName(); // name without path, will be passed to page via hidden-field
                        importFilePath = importFilename;
                    } else if (importFilePath.length()>0) { // file had been uploaded before
                        if (importTempfile!=null) {
                            File tempfile = new File(tempdir, importTempfile);
                            int filesize = (int)tempfile.length();
                            FileInputStream in = new FileInputStream(tempfile);
                            byte[] c = new byte[filesize];
                            in.read(c);
                            uploadString = new String(c, CHARACTER_ENCODING);
                            in.close();
                        }
                    }
                    if ((mode & MODE_SINGLE) != 0) {
                        Object o = parameters.get("single");
                        String[] singles;
                        if (o instanceof String) {
                            singles = new String[1];
                            singles[0]=(String)o;
                        } else {
                            singles = (String[])o;
                        }
                        if (singles != null) {
                            single = Arrays.asList(singles); //multiParam(parameters, "single");
                        }
                    }
                    
                    if ( uploadString != null ) {
                        ArchiveHandler io = (ArchiveHandler)ioHandlers.get(importParam);//getIOHandler(importParam);
                        Archive data = new Archive(); //createArchive();
                        Map attachments = new HashMap();
                        if (io != null) {
                            Object upload = io.createData();
                            if ( importzip ) {
                                uploadString = resolveZip(engine, uploadString, attachments);
                            }
                            io.stringToData(uploadString, upload);
                            io.performIO(true, data, upload, MODE_BOTH, null);
                        } else {
                            html.append(error("import type '"+importParam+"' unknown"));
                        }
                        importMessages = wikiIOHandler.performIO(false, data, new Object[] { engine, pagename, Boolean.valueOf(useOutgoingLinks) }, mode, single); // read data into Wiki
                        updateAttachments(engine, attachments); // ! TODO: will always update attachments, not only those among individually chosen entries
                    }
                }                 
            } else {            
                exportParam = context.getHttpParameter("export");
                createParam = context.getHttpParameter("create");
                deleteParam = enableDelete ? context.getHttpParameter("delete") : null;
                // -- Create --
                if ((createParam!=null)&&(createParam.length()>0)) {
                    StringBuffer sb = new StringBuffer();
                    StringTokenizer st = new StringTokenizer(createParam, " ,", false);
                    Collection c = cm.getCategoryMembers(pagename);
                    Collection l = new TreeSet(c);
                    while (st.hasMoreTokens()) {
                        String newpage = st.nextToken();
                        if ( ! l.contains(newpage)) {
                            CategoryManager.getInstance(engine).addCategoryMember(pagename, newpage);
                            sb.append(engine.textToHTML(context, "Leerer Eintrag ["+newpage+"] erzeugt.\\\\"));
                            
                            String url = engine.getEditURL(newpage);
                        } else {
                            sb.append(engine.textToHTML(context, "Eintrag ["+newpage+"] existiert bereits.\\\\"));
                        }
                    }
                    createMessages = sb.toString();
                // -- Export --
                // parameter "export" set? then generate data and send it later as raw ascii
                } else if (exportParam != null) {
                    mode = Integer.parseInt(context.getHttpParameter("exportmode"));
                    exportzip = (context.getHttpParameter("exportzip") != null);
                    String result = null;
                    if ((mode&MODE_SINGLE)==MODE_SINGLE) {
                        String[] ss = context.getHttpRequest().getParameterValues("single");
                        if (ss!=null) {
                            single = Arrays.asList(ss);
                        }
                    }
                    ArchiveHandler io = (ArchiveHandler)ioHandlers.get(exportParam);//getIOHandler(exportParam);
                    Archive archive = new Archive();//createArchive();
                    wikiIOHandler.performIO(true, archive, new Object[] { engine, pagename, Boolean.valueOf( useOutgoingLinks ) }, MODE_BOTH, null ); // extract data from Wiki
                    if (io != null) {
                        Object export = io.createData();
                        exportMessages = io.performIO(false, archive, export, mode, single); // export extracted data to output format
                        result = io.dataToString(export); //bibtex.toString();
                    } else {
                        html.append(error("export type '"+exportParam+"' unknown"));
                    }
                    if (( result != null) && ( (!((mode&MODE_SINGLE)==MODE_SINGLE)) || ((single!=null)&&(single.size()>0))) ) {
                        String suggestedFilename = pagename.toLowerCase() + io.getContentSuffix();
                        String contenttype;
                        if ( exportzip ) {
                            Collection pages = (single != null) ? single : cm.getCategoryMembers(pagename);
                            result = createZip(engine, result, suggestedFilename, pages);
                            contenttype = "application/zip";
                            suggestedFilename = pagename.toLowerCase() + ".zip";
                        } else {
                            contenttype = io.getContentType();
                        }
                        context.getHttpRequest().setAttribute(ATTRIBUTE_EXPORT_RESULT, result);
                        context.getHttpRequest().setAttribute(ATTRIBUTE_EXPORT_CONTENTTYPE, contenttype );
                        context.getHttpRequest().setAttribute(ATTRIBUTE_EXPORT_CONTENTDISPOSITION, "attachment; filename=" + suggestedFilename );
                        skipGUI = true;
                    }
                // -- Delete --
                } else if (deleteParam != null) {
                    String sure = context.getHttpParameter("sure");
                    if (sure!=null && sure.equals("yes")) {
                        Collection all = cm.getCategoryMembers(pagename); //getList(engine);
                        deletePages(engine, all);
                        html.append("<h3 align='center'><font color='red'>Das gesamte Archiv wurde gel&ouml;scht.</font></h3>");
                        deleteParam=null; // show normal gui
                    }
                }
            }
            // -- HTML GUI --  
            if (!skipGUI) {
                if (deleteParam==null) {
                    String categoryList = super.execute(context, params);
                    html.append("<div class='categoryMenu'>");
                    
                    boolean doImports = ( ! importFormats.isEmpty() );
                    boolean doExports = ( ! exportFormats.isEmpty() );
                    boolean doIO = doImports || doExports;
                    if ( ! useOutgoingLinks ) {
                        html.append(
                        "<p class='categoryMenuNew'>" +
                        heading("New entry") +
                        "<form method='GET'><table class='categoryMenu'><tr><td>" +
                        "<input type='hidden' name='page' value='"+thisPage+"'/>" +
                        "<input type='text' name='create' size='50'/>" +
                        (!doIO ? " <input type='submit' value='Add entry'/>" : "") + // Eintragen 
                        "<br/>" +
                        "<font size='-1'>(Multiple entries seperated by commas or spaces.)</font>" +
                        (doIO ? "</td></tr><tr><td><input type='submit' value='Add entry'/>" : "") + 
                        "</td></tr></table></form></p>");
                        if ( createMessages != null ) {
                            html.append("<br/>"+createMessages);
                        }
                        if ( doIO ) {
                            html.append("<hr class='categoryMenu'/>");
                        }
                    }
                    
                    // --- export ---------------------------------------------
                    if ( doExports ) {
                        if ( exportFormats.size() == 1 ) {
                            Iterator it = exportFormats.iterator();
                            singleFormat = (String)it.next();
                        } else {
                            singleFormat = null;
                        }
                        html.append(
                        "<p class='categoryMenuExport'>" +
                        "<a name='export'/>" +
                        heading("Export")
                        );
                        
                        html.append(
                        "<form action='templates/" + engine.getTemplateDir() + "/WikiArchiveExport.jsp' method='POST'>" + // !! WikiArchiveExport.jsp !!
                        "<input type='hidden' name='page' value='"+thisPage+"'/>" +
                        "<input type='hidden' name='exportmode' value='3'/>" +
                        "<table class='categoryMenu'>" +
                        "<tr><td width='100%'><input type='submit' value='"+(singleFormat!=null? "Export "+((String)formatLabels.get(singleFormat))+"" : "Export" )+"'/></td><td nowrap align='right'>" +
                        (singleFormat==null
                            ? "Format: "+selectbox("export", exportFormats, formatLabels, exportParam)+"<br/>"
                            : hiddenfield("export", singleFormat) ) +
                            
                        (enableZipAttachments
                            ? "<input type='checkbox' name='exportzip'"+(exportzip ? " checked" : "")+"/> .zip with attachments"
                            : "") +
                        "</td></tr></table>" +
                        "</form>"
                        );
                        html.append("</p>");
                    }
                    
                    // --- import ---------------------------------------------
                    if ( doImports ) {
                        if (importFormats.size() == 1) {
                            Iterator it = importFormats.iterator();
                            singleFormat = (String)it.next();
                        } else {
                            singleFormat = null;
                        }
                        html.append(
                        "<p class='categoryMenuImport'>" +
                        "<a name='import'/>\n" +
                        "<script language='JavaScript'>\n" +
                        "<!--\n" +
                        "function updateImportTypeSelectbox(filename) {\n" +
                        "    var pos = filename.lastIndexOf('.');\n" +
                        "    if (pos != -1) {\n" +
                        "        var suffix = filename.substring(pos + 1);\n" +
                        "        document.forms['importform'].elements['import'].value = suffix;" +
                        "    }\n" +
                        "}\n" +
                        "//--!>\n" +
                        "</script>\n" +
                        heading("Import") +
                        "<form action='?page="+thisPage+"#import' name='importform' enctype='multipart/form-data' method='POST' accept-charset='ISO-8859-1'>" +
                        "<input type='hidden' name='page' value='"+thisPage+"'/>" +
                        "<table class='categoryMenu'>" +
                        "<input type='hidden' name='importfilepath' value='"+((importFilePath!=null)?importFilePath:"")+"'/>" +
                        "<input type='hidden' name='importtempfile' value='"+((importTempfile!=null)?importTempfile:"")+"'/>" +
                        ( (importTempfile != null) ? "<tr><td>Datei:</td><td bgcolor='white'><b><code>"+( (importFilePath != null) ? importFilePath : importTempfile ) + "</code></b></td><td colspan='2'></td></tr>" : "") +
                        
                        "<tr><td>" + (importTempfile==null?"File:":"") + "</td><td width='100%'><input type='file' name='importfile' size='80' value='"+importFilePath+"' onChange='updateImportTypeSelectbox(this.value)'/></td><td nowrap align='right'>" +
                        (singleFormat==null
                            ? "Format: "+selectbox("import", importFormats, formatLabels, importParam)+"<br/>"
                            : hiddenfield("import", singleFormat) ) +
                        (enableZipAttachments
                            ? "<input type='checkbox' name='importzip'"+(importzip ? " checked" : "")+"/> .zip with attachments"
                            : "") +
                        "</td></tr>" +
                        "<tr><td valign='top'></td><td>" +
                        "<input type='radio' name='importmode' value='7'"+(mode==(MODE_BOTH|MODE_SINGLE)?" checked='true'":"")+"/>Confirm each entry individually<br/>" + // Mit Einzelbestaetigung
                        "<input type='radio' name='importmode' value='3'"+(mode==MODE_BOTH?" checked":"")+"/>Import all new and changed entries<br/>" + // Alle neuen Eintr&auml;ge und &Auml;nderungen &uuml;bernehmen
                        "<input type='radio' name='importmode' value='2'"+(mode==MODE_UPDATE?" checked":"")+"/>Import new entries (only display changed entries)<br/>" + // Neue Eintr&auml;ge &uuml;bernehmen (&Auml;nderungen nur anzeigen)
                        "<input type='radio' name='importmode' value='1'"+(mode==MODE_INSERT?" checked":"")+"/>Import changes (only display new entries)<br/>" + // &Auml;nderungen &uuml;bernehmen (neue Eintr&auml;ge nur anzeigen)
                        "<input type='radio' name='importmode' value='0'"+(mode==MODE_NONE?" checked":"")+"/>Display only, no import of new or changed entries" + //Nur anzeigen, keine neuen Eintr&auml;ge oder &Auml;nderungen &uuml;bernehmen
                        "</td><td></td></tr>" +
                        "<tr><td colspan='3'><input type='submit' value='"+( ( singleFormat != null ) ? "Import "+((String)formatLabels.get(singleFormat))+"" : "Import" )+"'/></td></tr>" +
                        "</table>" +
                        ( ( (importMessages != null) )  
                            ? ("<br/><br/>" + importMessages + ( ( ( mode & MODE_SINGLE ) != 0 ) 
                                                                ? "<br/><center><input type='submit' value='Import'/>&nbsp;&nbsp;&nbsp;<input type='button' value=' Cancel ' onClick='history.back()'/></center>" 
                                                                : "" ) )
                            : "" ) +
                        "<br/>" +
                        "</form>\n"
                        );
                        html.append("</p>");
                    }
                    html.append("</div>");
                    html.append("<p/>");
                    if (menuAtTop) {
                    	html.append(categoryList);
                        return html.toString();
                    } else {
                    	return categoryList + html.toString();
                    }
                } else {
                    // --- delete ---------------------------------------------
                    return "<h3 align='center'>Really DELETE EVERYTHING?</h3>" +
                           "<br/>" +
                           "<form>" +
                           "<input type='hidden' name='page' value='"+context.getPage().getName()+"'/>" +
                           "<center>" +
                           "<table class='categoryMenu'><tr><td align='center' valign='top' nowrap>" +
                           "<input type='submit' name='delete' value='Delete'/><br/>" +
                           "<br/>" +
                           "<input type='checkbox' name='sure' value='yes'/> <font color='red'>sure</font><br/>" +
                           "</td><td>" +
                           "&nbsp;&nbsp;&nbsp;" +
                           "</td><td align='center' valign='top' nowrap>" +
                           "<input type='submit' value='Cancel'/><br/>" +
                           "</td></tr></table>" +
                           "</center>" +
                           "</form>";
                }
            } else { // deletePages != null
                // just a message that would help to see that something is wrong, in case out.clear() later will fail for some reason:
                return "<font color='red'>(### Bibliography-plugin has generated export data, this message should NOT BE VISIBLE! ###)</font>";
            }
        } catch (Exception e) {
            throw new PluginException("wrapped exception: "+e.getClass().getName()+": "+e.getMessage()+"\n"+stackTrace(e), e);
        }
    } 

/**
 * 
 * 
 * @poseidon-object-id [Ie4883fm112767c1b91mm3781]
 * @return 
 * @param name 
 * @param wikipage 
 * @param engine 
 */
    protected String deriveFullTitle(String name, WikiPage wikipage, WikiEngine engine) {        
    	String title = null;
        if (wikipage != null) {
            String pagedata = engine.getPureText(name, WikiPageProvider.LATEST_VERSION);
        	ArchiveEntry entry = ArchiveHandlerWiki.wiki2entry(name, pagedata, null);
        	// this is a bit hacky, derive title by named entry (however, returning null will not cause a crash but is an allowed return value)
        	title = (String)entry.getMap().get("title"); // english TODO NO internationalization here
        	if (title == null) {
        		title = (String)entry.getMap().get("titel"); // german TODO internationalization here
        	}
        }
        return title;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7272]
 * @return 
 * @param th 
 */
    private static String stackTrace(Throwable th) {        
        StringWriter sw = new StringWriter();
        th.printStackTrace(new PrintWriter(sw));
        return sw.toString();
    } 

/**
 * Returns the property name for the specified sub-property.
 * This means that "jspwiki.<i>archivename</i>." will be preprended
 * to the propname.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm726b]
 * @return The full property name for reading from jspwiki.properties.
 * @param propname The sub-property name.
 * @see #name
 */
    public String getPropertyName(String propname) {        
        return "jspwiki.archive." + propname;
    } 

/**
 * Returns a property entry from the JSPWiki configuration file.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7260]
 * @return The property value, or null if it does not exists.
 * @param engine The WikiEngine.
 * @param propname The propertyname, without leading "jspwiki.<archivename>.".
 */
    public String getProperty(WikiEngine engine, String propname) {        
        return getProperty( engine, propname, null );
    } 

/**
 * Returns a property entry from the JSPWiki configuration file.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7254]
 * @return The property value, or null if it does not exists.
 * @param engine The WikiEngine.
 * @param propname The propertyname, without leading "jspwiki.<archivename>.".
 * @param defaultValue 
 */
    public String getProperty(WikiEngine engine, String propname, String defaultValue) {        
        String v = engine.getWikiProperties().getProperty(getPropertyName(propname));
        if ( v == null ) {
            return defaultValue;
        } else {
            return v;
        }
    } 

/**
 * Removes a collection of pages from the Wiki. Files get deleted in the filesystem.
 * This should have no disturbing effect on the running Wiki, but to be sure
 * use this operation only when the Wiki can be manually restarted afterwards.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7249]
 * @param engine The WikiEngine.
 * @param pageNames The names of pages to be deleted.
 */
    protected void deletePages(WikiEngine engine, Collection pageNames) throws ProviderException {        
        String pagedirname = engine.getWikiProperties().getProperty("jspwiki.fileSystemProvider.pageDir");
        File pagedir = new File(pagedirname);
        for (Iterator it = pageNames.iterator(); it.hasNext(); ) {
            String p = (String)it.next();
            // delete 'physical' file (not done by PageManager, which seems to be considered a bug there (v. 2.0.52))
            String pEnc = engine.encodeName(p); // ensure that umlauts are translated to %xx
            File f = new File(pagedir, pEnc+".txt");
            boolean del = f.delete();
            if (del) {
                System.out.println("ARCHIVE DELETE: file '"+f.getAbsolutePath()+"' has been deleted");
            }
            // forget in cache
            engine.getPageManager().getProvider().deletePage(p);
        }
    } 

/**
 * Handles file downloads to the client.
 * To be invoked by a Servlet/JSP after any other part of the page has been generated.
 * If a download is to be performed, any previously generated page content will be removed.
 * (See end of ViewTemplate.jsp.)
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm723d]
 * @param request The http-request.
 * @param response The http-response.
 * @param out The page output writer.
 * @throws IOException if an i/o error occurs
 */
    public static void pageEnd(HttpServletRequest request, HttpServletResponse response, JspWriter out) throws IOException {        
        // Send export data if such has been generated.
        String export = (String)request.getAttribute( ATTRIBUTE_EXPORT_RESULT );
        if (export != null) {
            String exportContentType = (String)request.getAttribute( ATTRIBUTE_EXPORT_CONTENTTYPE );
            String exportContentDisposition = (String)request.getAttribute( ATTRIBUTE_EXPORT_CONTENTDISPOSITION );
            out.clear(); // IMPORTANT: must not have been flushed before
            response.setContentType(exportContentType);
            if ((exportContentDisposition != null)&&(response instanceof javax.servlet.http.HttpServletResponse)) {
                ((javax.servlet.http.HttpServletResponse)response).setHeader("Content-Disposition", exportContentDisposition);
            }
            response.setContentLength(export.length());
            out.print(export);
        }
    } 

/**
 * Formats a string as HTML-warning message.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7228]
 * @return The formatted HTML string.
 * @param s The string to be formatted.
 */
    protected static String warning(String s) {        
        return "<h4>"+s+"</h4>";
    } 

/**
 * Formats a string as HTML-error message.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm721f]
 * @return The formatted HTML string.
 * @param s The string to be formatted.
 */
    protected static String error(String s) {        
        return "<h4 align='center'><font color='red'>Error: "+s+"</font></h4>";
    } 

/**
 * Outputs an HTML-form-element for a select-box.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7213]
 * @return 
 * @param name The name of the select-box element.
 * @param keys The value keys that are selectable by the select-box.
 * @param labelsByKey A map assigning displayable labels bto each value key.
 * The are the option texts presented to the user.
 * @param value The current value to be pre-selected.
 */
    protected static String selectbox(String name, Collection keys, Map labelsByKey, String value) {        
        StringBuffer sb = new StringBuffer("<select name='"+name+"'>");
        for (Iterator it = keys.iterator(); it.hasNext(); ) {
            String key = (String)it.next();
            String sel;
            if ( (value != null) && key.equals(value) ) {
                sel = " selected='true'";
            } else {
                sel = "";
            }
            sb.append("<option value='"+key+"'"+sel+">"+((String)labelsByKey.get(key))+"</option>");
        }
        sb.append("</select>");
        return sb.toString();
    } 

/**
 * Outputs an HTML-form-element for a hidden-field.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7207]
 * @return 
 * @param name The name of the field.
 * @param value The value of the field.
 */
    protected static String hiddenfield(String name, String value) {        
        return "<input type='hidden' name='"+name+"' value='"+value+"'>";
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm71fb]
 * @return 
 * @param engine 
 * @param dataString 
 * @param filename 
 * @param pages 
 */
    public static String createZip(WikiEngine engine, String dataString, String filename, Collection pages) throws IOException, ProviderException {        
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        ZipOutputStream zip = new ZipOutputStream(out);
        ZipEntry entry = new ZipEntry(filename);        
        zip.putNextEntry(entry);
        zip.write(dataString.getBytes(CHARACTER_ENCODING));
        AttachmentManager am = engine.getAttachmentManager();
        for (Iterator it = pages.iterator(); it.hasNext(); ) {
            String page = (String)it.next();
            WikiPage p = engine.getPage(page);
            Collection atts = am.listAttachments(p);
            for (Iterator jt = atts.iterator(); jt.hasNext(); ) {
                Attachment att = (Attachment)jt.next();
                InputStream in = am.getAttachmentStream(att);
                byte[] b = WikiToolbox.readInputStream(in);
                in.close();
                entry = new ZipEntry(att.getName());
                zip.putNextEntry(entry);
                zip.write(b);
            }
        }
        zip.close();
        String zipString = new String(out.toByteArray(), CHARACTER_ENCODING);
        return zipString;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm71f0]
 * @return 
 * @param engine 
 * @param dataString 
 * @param attachmentsToBeFilled 
 */
    public static String resolveZip(WikiEngine engine, String dataString, Map attachmentsToBeFilled) throws IOException {        
        String unzipString = null;
        ByteArrayInputStream in = new ByteArrayInputStream(dataString.getBytes(CHARACTER_ENCODING));
        ZipInputStream zip = new ZipInputStream(in);
        ZipEntry entry = zip.getNextEntry();
        while ( entry != null ) {
            String name = entry.getName();
            byte[] b = WikiToolbox.readInputStream(zip);
            if ( name.indexOf('/') == -1 ) {
                unzipString = new String(b, CHARACTER_ENCODING);
            } else {
                attachmentsToBeFilled.put(name, b);
            }
            entry = zip.getNextEntry();
        }
        zip.close();
        return unzipString; // TODO
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm71e7]
 * @param engine 
 * @param attachments 
 */
    protected static void updateAttachments(WikiEngine engine, Map attachments) throws IOException, ProviderException {        
        for (Iterator it = attachments.entrySet().iterator(); it.hasNext(); ) {
            Map.Entry entry = (Map.Entry)it.next();
            String path = (String)entry.getKey();
            byte[] attachmentData = (byte[])entry.getValue();
            updateAttachment(engine, path, attachmentData);
        }
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm71da]
 * @param engine 
 * @param path 
 * @param data 
 */
    protected static void updateAttachment(WikiEngine engine, String path, byte[] data) throws IOException, ProviderException {        
        int pos = path.indexOf('/');
        String page = path.substring(0, pos);
        String filename = path.substring(pos+1);
        AttachmentManager am = engine.getAttachmentManager();
        Attachment att = am.getAttachmentInfo(path);
        InputStream old = am.getAttachmentStream(att);
        byte[] b = null;
        if ( old != null ) {
            b = WikiToolbox.readInputStream(old);
        }
        boolean replace;
        if ( b != null ) {
            if ( b.length == data.length ) {
                replace = false;
                for ( int i = 0; ( ! replace ) && (i < data.length); i++ ) {
                    replace = ( b[i] != data[i] );
                }
            } else { // differ in size
                replace = true;
            }
        } else { // not yet there
            replace = true;
        }
        if ( replace ) { // ignore if current attachment is exactly the same
            InputStream in = new ByteArrayInputStream(data);
            am.storeAttachment(att, in);
        }
    } 
 }
