/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive;

import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import java.util.*;

/**
 * An archive representation, that means a set of <code>ArchiveEntry</code>s together with a sort order.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e4a]
 * @author Jens Gulden
 */
public class Archive extends TreeMap implements Comparator {

/**
 * 
 * 
 * @poseidon-object-id [I724519c4m1127a063bbemm368d]
 */
    private static final long serialVersionUID = 1l;

/**
 * Adds an entry to the archive.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7989]
 * @param entry The entry.
 */
    public void add(ArchiveEntry entry) {        
        this.put(entry.getName(), entry);
    } 

/**
 * Returns the comparator to be used for comparing pairs of entries
 * and building a sorted entry list.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7982]
 * @return This archive's default entry-comparator.
 */
    public Comparator getComparator() {        
        return this;
    } 

/**
 * Compares two ArhiveEntries with each other.
 * This is done based on an case-insensitive comparison of the entry-names.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7975]
 * @return 
 * @param o1 One ArchiveEntry.
 * @param o2 Another ArchiveEntry.
 */
    public int compare(Object o1, Object o2) {        
        if (o1 instanceof ArchiveEntry) {
            return ((ArchiveEntry)o1).compareTo(o2);
        } else {
            return o1.toString().toUpperCase().compareTo(o2.toString().toUpperCase());
        }
    } 
 }
