/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive;

import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive.ArchiveEntryImpl;
import de.tuberlin.cs.flp.jspwiki.util.WikiToolbox;
import java.util.*;

/**
 * A bibliography entry.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e49]
 * @author Jens Gulden
 */
public class ArchiveEntryBibTeX extends ArchiveEntryImpl {

/**
 * 
 * 
 * @poseidon-object-id [I724519c4m1127a063bbemm3469]
 */
    private static final long serialVersionUID = 1l;
/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm75f1]
 */
    protected ArchiveHandlerBibTeX archiveHandler;

/**
 * Creates a new instance of BibEntry.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm75e8]
 * @param archiveHandler 
 */
    public  ArchiveEntryBibTeX(ArchiveHandlerBibTeX archiveHandler) {        
        super();
        this.archiveHandler = archiveHandler;
    } 

/**
 * Creates a new instance of BibEntry.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm75df]
 * @param archiveHandler 
 * @param name 
 * @param m Map with initial field values.
 */
    public  ArchiveEntryBibTeX(ArchiveHandlerBibTeX archiveHandler, String name, Map m) {        
        super(name, m);
        this.archiveHandler = archiveHandler;
        setName(name);
    } 

/**
 * Returns the BibTeX-type of this entry.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm75d8]
 * @return Type in BibTeX naming.
 */
    public String getType() {        
        String t = getValue("type");
        if (t.length()!=0) {
            return t;
        } else {
            return "unknown";
        }
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm75d1]
 * @return 
 */
    public boolean isOptional() {        
        return archiveHandler.isEntryOptional(this.getName());
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm75c8]
 * @return 
 * @param key 
 * @param val1 
 * @param val2 
 */
    protected boolean equalEntry(String key, String val1, String val2) {        
        String keyTranslated = archiveHandler.translateKey(key, true);
        if ( keyTranslated.equals("type") ) {
            return val1.equalsIgnoreCase(val2); // ignore case
        } else {
            return super.equalEntry(key, val1, val2);
        }
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm75c2]
 * @return 
 */
    public String toString() {        
        // find entry format string
        String type = getType();
        Properties formats = archiveHandler.getFormats();
        String f = formats.getProperty(type, formats.getProperty("default")); //FORMAT[0][1]; // first one is default
        Map m = archiveHandler.translateKeys(this, true);
        m.put("name", getName());
        return WikiToolbox.format(m, f);
    } 

    /** @poseidon-generated */
    public ArchiveHandlerBibTeX getArchiveHandler() {
        return archiveHandler;
    }
    /** @poseidon-generated */
    public void setArchiveHandler(ArchiveHandlerBibTeX archiveHandlerBibTeX) {
        this.archiveHandler = archiveHandlerBibTeX;
    }
 }
