/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive;

import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive.ArchiveEntry;
import de.tuberlin.cs.flp.jspwiki.util.PropertiesOrdered;
import java.util.*;

/**
 * Default implementation of an ArchiveEntry. Extends a HashMap to allow storage of individual key/value-fields as entry content.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e30]
 * @author Jens Gulden
 */
public class ArchiveEntryImpl extends PropertiesOrdered implements ArchiveEntry {

/**
 * 
 * 
 * @poseidon-object-id [I724519c4m1127a063bbemm3667]
 */
    private static final long serialVersionUID = 1l;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7967]
 */
    protected String name;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7962]
 */
    protected String text = null;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm795c]
 */
    protected String plaintext = null;

/**
 * Creates a new instance of ArchiveEntryImpl.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7955]
 */
    public  ArchiveEntryImpl() {        
        super();
    } 

/**
 * Creates a new instance of ArchiveEntryImpl.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm794d]
 * @param name 
 * @param m Map with default key/value-fields.
 */
    public  ArchiveEntryImpl(String name, Map m) {        
        this(name);
        this.putAll(m);
    } 

/**
 * Creates a new instance of ArchiveEntryImpl.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7945]
 * @param name The name of the entry.
 */
    public  ArchiveEntryImpl(String name) {        
        this();
        setName(name);
    } 

/**
 * Returns the name of the entry.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm793e]
 * @return The name.
 */
    public String getName() {        
        return name;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7936]
 * @param name 
 */
    public void setName(String name) {        
        this.name = name;
    } 

/**
 * Gets a field's value.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm792f]
 * @return The value, or an empty string if the value as not been set. Never returns null.
 * @param key The field key.
 */
    public String getValue(String key) {        
        String v = (String)get(key);
        if (v != null) {
            return v;
        } else {
            return "";
        }
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7927]
 * @return 
 */
    public String getText() {        
        return text;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7920]
 * @param text 
 */
    public void setText(String text) {        
        this.text = text;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm791a]
 * @return 
 */
    public String getPlainText() {        
        return plaintext;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7913]
 * @param text 
 */
    public void setPlainText(String text) {        
        this.plaintext = text;
    } 

/**
 * May be overwritten by subclass.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm790c]
 * @return 
 */
    public boolean isOptional() {        
        return false;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7906]
 * @return 
 */
    public Map getMap() {        
        return this;
    } 

/**
 * May be overwritten by subclass.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7900]
 * @return 
 */
    public String toString() {        
        return getName();
    } 

/**
 * Compares the name
 * This can be used for implenting a Comparator and using it to sort a list of entry names.
 * The comparison is performed case-insensitive.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm78f9]
 * @return A value &lt;0 if this entrie's name is to be sorted before the other entry,<br>
 * 0 if the names are equal,<br>
 * a value &gt;0 if this entrie's name is to be sorted behind the other entry.<br>
 * @param o An instance of <code>ArchiveEntry</code> to compare with based on the name.
 */
    public int compareTo(Object o) {        
        if (o instanceof ArchiveEntry) {
            return this.getName().toUpperCase().compareTo(((ArchiveEntry)o).getName().toUpperCase());
        } else {
            return 1;
        }
    } 

/**
 * Tests for equality of the whole entry. Unlike compareTo(),
 * this compares all key/value-fields, too, not only the entry name.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm78f0]
 * @return <emp>true</emp> if the entries and all their fields are equal, <emp>false</emp> otherwise.
 * @param o Another instance of <code>ArchiveEntryImpl</code> to test for equality with.
 */
    public boolean equals(Object o) {        
        if ((o instanceof ArchiveEntryImpl) && (compareTo(o)==0)) {
            ArchiveEntryImpl other = (ArchiveEntryImpl)o;
            for (Iterator it = this.entrySet().iterator(); it.hasNext(); ) {
                Map.Entry e = (Map.Entry)it.next();
                String key = (String)e.getKey();
                String val = (String)e.getValue();
                String v = (String)other.get( key );
                if (v != null) {
                    if ( val == null ) {
                        val = "";
                    }
                    v = v.trim();
                    if ( ! ( equalEntry(key, val, v) || equalEntry(key, v, val) ) ) { // (do symmetric, in order to allow any subclass to lossen equalslity cond. on single entries)
                        return false;
                    }
                } else { // v==null
                    if ( (val != null) && (val.trim().length() > 0) ) {
                        return false;
                    } else {
                        other.put(key, ""); // null could mean missing element: make sure that entry exists in other map (so strict equality will be guaranteed after equals() returns - this is a side effect)
                    }
                }
            }
            return true;
        } else {
            return false;
        }
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm78e5]
 * @return 
 * @param key 
 * @param val1 
 * @param val2 
 */
    protected boolean equalEntry(String key, String val1, String val2) {        
        return val1.equals( val2 );
    } 
 }
