/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive;

import com.ecyrd.jspwiki.WikiEngine;
import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import java.io.*;
import java.util.*;
/**
 * Interface ArchiveIOHandler. Each i/o-handler takes care about import and export between the <code>Archive</code> model interface and one type of data source/sink. Importing and exporting from/to external data sources is implicitly implemented by WikiArchive, which uses the <code>Archive</code> model interface as intermediate data container between the Wiki's internal i/o-handler and any other i/o-handler. Using this architecture, only bidirectional conversions between the <code>Archive</code> model interface and each data source (including the Wiki itself) need to be implemented. Direct conversions between different types are never needed, as the <code>Archive</code> model interface acts as intermediate data container. Each WikiArchive needs at least one ArchiveIOHandler registered for type &quot;wiki&quot;, Allows the implementation of pluggable i/o-handlers for internal Wiki i/o and any external import/export data source.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e38]
 * @author Jens Gulden
 * @see de.tuberlin.cs.flp.jspwiki.plugin.WikiArchive#initIOHandlers(Object[][])
 */
public interface ArchiveHandler {

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm75b8]
 */
    public static final boolean IMPORT = false;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm75b2]
 */
    public static final boolean EXPORT = true;
/**
 * Initializes this ArchiveHandler.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm75a9]
 * @param engine 
 * @param config 
 */
    public void init(WikiEngine engine, Map config);
/**
 * Perform either input from the <code>Archive</code> model interface
 * into the data source handled by this ArchiveIOHandler.
 * @poseidon-object-id [Im7d594a32m112753776a4mm759e]
 * @param plugin The WikiArchive plugin.
 * 
 * @return 
 * @param io Switch between input or output mode, <emp>true</emp>
 * for input into data source, <emp>false</emp>
 * for output into <code>Archive</code> model interface.
 * @param archive The <code>Archive</code> model interface.
 * @param data The data source handled by this ArchiveIOHandler.
 * @param mode The processing mode. This is a bit mask with:<br>
 * bit #0 set: update exisiting entries with new values, if new data is different<br>
 * bit #1 set: insert new entries<br>
 * bit #2 set: ask individually for each entry (generate HTML checkbox-form first)
 * @param exceptions Collection of entry names that should be treated as exceptions to the behaviour specified by <code>mode</code>. (Not used.)
 * @throws IOException if an i/o error occurs
 */
    public String performIO(boolean io, Archive archive, Object data, int mode, Collection exceptions) throws IOException;
/**
 * Creates an empty data source to be filled for export.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7591]
 * @return 
 */
    public Object createData() throws IOException;
/**
 * Converts the handled data source to string for export.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7589]
 * @return 
 * @param data 
 */
    public String dataToString(Object data) throws IOException;
/**
 * Converts a string to a handled data source for import.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7580]
 * @param s 
 * @param data 
 */
    public void stringToData(String s, Object data) throws IOException;
/**
 * Content type for export via HTTP.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7579]
 * @return 
 */
    public String getContentType();
/**
 * Content suffix for export via HTTP.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7573]
 * @return 
 */
    public String getContentSuffix();
}


