/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive;

import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import de.tuberlin.cs.flp.jspwiki.plugin.WikiArchive;
import java.io.IOException;

/**
 * Common superclass for archive handlers that interface to an external data format.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e3a]
 * @author Jens Gulden
 */
public abstract class ArchiveHandlerExternalFormatAbstract extends ArchiveHandlerAbstract {

/**
 * Creates an empty data source to be filled for export.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm74c3]
 * @return 
 */
    public Object createData() throws IOException {        
        // default, may be overwritten by subclass
        return new StringBuffer();
    } 

/**
 * Converts data source to string for export.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm74bb]
 * @return 
 * @param data 
 */
    public String dataToString(Object data) throws IOException {        
        // default, may be overwritten by subclass
        return data.toString();
    } 

/**
 * Converts a string to a handled data source for import.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm74b2]
 * @param s 
 * @param data 
 */
    public void stringToData(String s, Object data) throws IOException {        
        // default, may be overwritten by subclass
        StringBuffer sb = (StringBuffer)data;
        sb.append(s);
    } 

/**
 * Outputs a message informing about the action that has been taken for an entry.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm74a8]
 * @return The message string. (Actually used to build a part of an HTML-form user interface.)
 * @param entry The entry processed.
 * @param outEntry The currently existing possible output entry, or null if such does not exist yet.
 * @param mode The action mode, like the <code>mode</code> parameter of performIO().
 */
    protected String msgDone(ArchiveEntry entry, ArchiveEntry outEntry, int mode) {        
        switch (mode) {
            case WikiArchive.MODE_INSERT:
                return "<td align='center'><font face='Helvetica,Arial' size='1'>Ok</font></td>";
            case WikiArchive.MODE_UPDATE:
                return "";
            default:
                return null;
        }
    } 

/**
 * Outputs a message informing that an action has not been taken for an entry.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm749b]
 * @return The message string. (Actually used to build a part of an HTML-form user interface.)
 * @param entry The entry not processed.
 * @param outEntry The currently existing possible output entry, or null if such does not exist yet.
 * @param mode The action mode, like the <code>mode</code> parameter of performIO().
 */
    protected String msgNotDone(ArchiveEntry entry, ArchiveEntry outEntry, int mode) {        
        if ((mode & WikiArchive.MODE_SINGLE) != 0) {
            switch (mode & WikiArchive.MODE_BOTH) {
                case WikiArchive.MODE_INSERT:
                    return "<td>&nbsp;&nbsp;&nbsp;<input type='checkbox' name='single' value='"+entry.getName()+"'/>&nbsp;&nbsp;&nbsp;</td>";
                case WikiArchive.MODE_UPDATE:
                    return "";
                default:
                    return null;
            }            
        } else {
            return "";
        }
    } 

/**
 * Outputs a message informing about the possible action to be taken for an input entry, e.g.
 * whether it would be added as a new entry, or an existing one would be updated.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm748e]
 * @return The message string. (Actually used to build a part of an HTML-form user interface.)
 * @param inEntry The input entry.
 * @param outEntry The currently existing possible output entry, or null if such does not exist yet.
 * @param mode The action mode, like the <code>mode</code> parameter of performIO().
 */
    protected String msgToDo(ArchiveEntry inEntry, ArchiveEntry outEntry, int mode) {        
        switch (mode) {
            case WikiArchive.MODE_INSERT:
                return "<td bgcolor='#dddddd' width='100%'>" +
                       "<table width='100%'><tr><td bgcolor='#ffffff' width='100%'>" +
                       "<font size='-1'>" +
                       inEntry.toString() +
                       "</font>" +
                       "</td></tr></table>" +
                       "</td>";
            case WikiArchive.MODE_UPDATE:
                return "";
            case WikiArchive.MODE_NONE:
                return "";
            default:
                return null;
        }
    } 
 }
