/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive;

import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import de.tuberlin.cs.flp.jspwiki.util.WikiToolbox;
import java.io.IOException;
import java.util.*;

/**
 * Archive handler for archives which treat their entries as plain text.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e36]
 * @author Jens Gulden
 */
public class ArchiveHandlerPlaintext extends ArchiveHandlerExternalFormatAbstract {

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7482]
 */
    public static final String CONTENT_TYPE = "text/plain; charset=ISO-8859-1";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm747c]
 */
    public static final String CONTENT_SUFFIX = ".txt";

/**
 * Start and end of an entry name, introducing a new entry.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7476]
 */
    protected static final String ENTRY_MARKER = "===";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7470]
 */
    protected static final char ENTRY_MARKER_CHAR = '=';

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm746a]
 */
    protected static final int MARKER_SIZE = 79;

/**
 * Content type for export via HTTP.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7463]
 * @return 
 */
    public String getContentType() {        
        return CONTENT_TYPE;
    } 

/**
 * Content type for export via HTTP.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm745d]
 * @return 
 */
    public String getContentSuffix() {        
        return CONTENT_SUFFIX;
    } 

/**
 * Creates a new archive entry in the data source handled by this ArchiveIOHandler.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7455]
 * @param data The data source handled by this ArchiveHandler, which is of type StringBuffer here.
 * @param entry The entry.
 */
    protected void createEntry(Object data, ArchiveEntry entry) {        
        StringBuffer buf = (StringBuffer)data;
        buf.append(entry2text(entry));
    } 

/**
 * Replaces an archive entry in the data source handled by this ArchiveIOHandler.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm744a]
 * @param data The data source handled by this ArchiveHandler, which is of type StringBuffer here.
 * @param newEntry The new entry.
 * @param oldEntry The old entry.
 */
    protected void replaceEntry(Object data, ArchiveEntry newEntry, ArchiveEntry oldEntry) {        
        StringBuffer buf = (StringBuffer)data;
        String old = entry2text(oldEntry);
        int pos = buf.indexOf(old);
        if (pos != -1) {
            buf.delete(pos, pos+old.length());
            buf.insert(pos, entry2text(newEntry));
        } else {
            createEntry(data, newEntry);
        }
    } 

/**
 * Retrieves all archive entries from the data source.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7440]
 * @return 
 * @param data The data source handled by this ArchiveHandler, which is of type StringBuffer here.
 * @throws IOException if an i/o error occurs
 */
    protected Map parseAll(Object data) throws IOException {        
        Map result = new HashMap(); // TreeMap better ? (see ArchiveHandlerBibTeX)
        String txt = data.toString();
        Collection lines = WikiToolbox.splitLines(txt);
        String currentEntry = null;
        StringBuffer sb = new StringBuffer();
        for (Iterator it = lines.iterator(); it.hasNext(); ) {
            String line = (String)it.next();
            String lineTrim = line.trim();
            if ( lineTrim.startsWith( ENTRY_MARKER ) ) {
                lineTrim = lineTrim.substring( ENTRY_MARKER.length() );
                //lineTrim = lineTrim.trim();
                int endpos = lineTrim.indexOf( ENTRY_MARKER );
                if (endpos == -1) {
                    endpos = lineTrim.length();
                }
                if (currentEntry != null) {
                    ArchiveEntryImpl entry = new ArchiveEntryPlaintext(currentEntry);
                    String text = sb.toString().trim();
                    entry.setText(text);
                    entry.setPlainText(text);
                    result.put(currentEntry, entry);
                }
                currentEntry = lineTrim.substring(0, endpos).trim();
                currentEntry = WikiToolbox.normalizeName(currentEntry);
                sb = new StringBuffer();
            } else {
                sb.append( line ).append('\n');
            }
        }
        return result;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7436]
 * @return 
 * @param entry 
 */
    protected static String entry2text(ArchiveEntry entry) {        
        String name = entry.getName();
        String text = entry.getPlainText();
        String entr = ENTRY_MARKER + " " + name + " " + ENTRY_MARKER;
        int l = entr.length();
        entr += WikiToolbox.repeat( MARKER_SIZE - l, ENTRY_MARKER_CHAR);
        entr += "\n\n" + text + "\n";
        return entr;
    } 
 }
