/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive;

import com.ecyrd.jspwiki.*;
import com.ecyrd.jspwiki.providers.*;
import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import de.tuberlin.cs.flp.jspwiki.plugin.WikiArchive;
import de.tuberlin.cs.flp.jspwiki.util.*;
import java.io.IOException;
import java.util.*;

/**
 * I/O-handler for bidirectional data interchange from/to Wiki pages. This is the 'internal' i/o-handler of the Bibliography Wiki-archive.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e3e]
 * @author Jens Gulden
 */
public class ArchiveHandlerWiki extends ArchiveHandlerAbstract {

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7745]
 */
    protected String categoryMarker;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm773e]
 * @param categoryMarker 
 */
    public  ArchiveHandlerWiki(String categoryMarker) {        
        super();
        this.categoryMarker = categoryMarker;
    } 

/**
 * Creates an empty data source to be filled for export.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7738]
 * @return 
 */
    public Object createData() {        
        return null; // not used with ArchiveHandlerWiki
    } 

/**
 * Converts the handled data source to string for export.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7731]
 * @return 
 * @param data 
 */
    public String dataToString(Object data) {        
        return null; // not used with ArchiveHandlerWiki
    } 

/**
 * Converts a string to a handled data source for import.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7729]
 * @param s 
 * @param data 
 */
    public void stringToData(String s, Object data) {        
        //nop, not used with ArchiveHandlerWiki
    } 

/**
 * Content type for export via HTTP.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7723]
 * @return 
 */
    public String getContentType() {        
        return null; // not used with ArchiveHandlerWiki
    } 

/**
 * Content type for export via HTTP.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm771d]
 * @return 
 */
    public String getContentSuffix() {        
        return null; // not used with ArchiveHandlerWiki
    } 

/**
 * Retrieves all archive entries from the data source.
 * ("Parse" is not the best chosen term in this method's name, although
 * in some cases it may perform parsing if the data source is a string.)
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7716]
 * @return 
 * @param data The data source handled by this ArchiveIOHandler.
 */
    protected Map parseAll(Object data) {        
        Object[] o = (Object[])data;
        WikiEngine engine = (WikiEngine)o[0];
        String category = (String)o[1];
        boolean useOutgoingLinks = ((Boolean)o[2]).booleanValue();
        HashMap result = new HashMap();
        // get all links from bibliography page to other pages (these are the actual bibliography entries)
        Collection links;
        CategoryManager cm = CategoryManager.getInstance(engine);
        if ( ! useOutgoingLinks ) {
            links = cm.getCategoryMembers(category); // usual: use category member
        } else {
            links = WikiToolbox.parseWikiLinks(engine.getText(category)); // special: export directly from page-links
        }
        for (Iterator it = links.iterator(); it.hasNext(); ) {
            String link = (String)it.next();
            if ( link.indexOf('/') == -1 ) { // just to make sure it's a normal page (might be attachment when converting legacy archives using 'usePageLinks' parameters)
                WikiPage page = engine.getPage(link);
                if (page != null) {
                    String pagedata = engine.getPureText(page.getName(), WikiPageProvider.LATEST_VERSION);
                    ArchiveEntry entry = wiki2entry(page.getName(), pagedata, categoryMarker); //, useOutgoingLinks);
                    result.put(entry.getName(), entry);
                }
            }
        }
        return result;
    } 

/**
 * Creates a new archive entry in the data source handled by this ArchiveIOHandler.
 * @poseidon-object-id [Im7d594a32m112753776a4mm770d]
 * @param plugin The WikiArcive plugin.
 * 
 * @param data The data source handled by this ArchiveIOHandler.
 * @param entry The entry.
 * @throws IOException if an i/o error occurs
 */
    protected void createEntry(Object data, ArchiveEntry entry) throws IOException {        
        Object[] o = (Object[])data;
        WikiEngine engine = (WikiEngine)o[0];
        String category = (String)o[1];
        boolean useOutgoingLinks = ((Boolean)o[2]).booleanValue();
        // create new Wiki page
        String name = entry.getName();
        WikiPage page = new WikiPage(engine, name);
        try {
            String pagetext = entry2wiki(entry, categoryMarker, useOutgoingLinks);
            PageManager pm = engine.getPageManager();
            pm.putPageText(page, pagetext);
            engine.getHTML(name); // needed to reindex?
            engine.getSearchManager().reindexPage(page); // ??? BUG: DOES NOT WORK - PAGE WILL ONLY BE VISIBLE AFTER RESTART OF SERVER!
            CategoryManager cm = CategoryManager.getInstance(engine);
            cm.addCategoryMember(category, name); // WORKAROUND: make sure at least a link to new entries appears
        } catch (ProviderException pe) {
            throw new IOException(pe.getMessage());
        }
    } 

/**
 * Replaces an archive entry in the data source handled by this ArchiveIOHandler.
 * @poseidon-object-id [Im7d594a32m112753776a4mm7700]
 * @param plugin The WikiArcive plugin.
 * 
 * @param data The data source handled by this ArchiveIOHandler.
 * @param newEntry The new entry.
 * @param oldEntry The old entry.
 * @throws IOException if an i/o error occurs
 */
    protected void replaceEntry(Object data, ArchiveEntry newEntry, ArchiveEntry oldEntry) throws IOException {        
        createEntry(data, newEntry);
    } 

/**
 * Outputs a message informing about the possible action to be taken for an input entry, e.g.
 * whether it would be added as a new entry, or an existing one would be updated.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm76f2]
 * @return The message string. (Actually used to build a part of an HTML-form user interface.)
 * @param inEntry The input entry.
 * @param outEntry The currently existing possible output entry, or null if such does not exist yet.
 * @param mode The action mode, like the <code>mode</code> parameter of performIO().
 */
    protected String msgToDo(ArchiveEntry inEntry, ArchiveEntry outEntry, int mode) {        
        switch (mode) {
            case WikiArchive.MODE_INSERT:
                return "<td bgcolor='lightgreen' width='100%'>" + 
                       "<table width='100%'><tr><td bgcolor='#ffffff' width='100%'>" +
                       "<font size='-1'>" +
                       inEntry.toString() +
                       "</font>" +
                       "</td></tr></table>" +
                       "</td>" +
                       "<td bgcolor='lightgreen' align='center'><font face='Helvetica,Arial' size='1'>neu</font></td>";
            case WikiArchive.MODE_UPDATE:
                return "<td bgcolor='lightblue' width='100%'>" + 
                       "<table width='100%'>" +
                       "<tr><td bgcolor='#ffffff' width='100%'>" +
                       "<font size='-1'>" +
                       inEntry.toString() +
                       "</font>" +
                       "</td></tr>" +
                       "<tr><td bgcolor='#aaaaaa'>" +
                       "<font size='-1'>" +
                       inEntry.toString() +
                       "</font>" +
                       "</td></tr>" +
                       "</table>" +
                       "</td>" +
                       "<td bgcolor='lightblue' align='center'><font face='Helvetica,Arial' size='1'>ge&auml;ndert</font></td>";
            case WikiArchive.MODE_NONE:
                return "<td bgcolor='#aaaaaa'>&nbsp;</td>" + // (!)
                       "<td>" + 
                       "<table width='100%'><tr>" +
                       "<td bgcolor='#dddddd' width='100%'>" +
                       "<font size='-1'>" +
                       inEntry.toString() +
                       "</font>" +
                       "</td></tr></table>" +
                       "</td>" +
                       "<td bgcolor='#aaaaaa' align='center'><font face='Helvetica,Arial' size='1'>unver&auml;ndert</font></td>";
            default:
                return null;
        }
    } 

/**
 * Outputs a message informing about the action that has been taken for an entry.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm76e5]
 * @return The message string. (Actually used to build a part of an HTML-form user interface.)
 * @param inEntry The entry processed.
 * @param outEntry The currently existing possible output entry, or null if such does not exist yet.
 * @param mode The action mode, like the <code>mode</code> parameter of performIO().
 */
    protected String msgDone(ArchiveEntry inEntry, ArchiveEntry outEntry, int mode) {        
        
        switch (mode) {
            case WikiArchive.MODE_INSERT:
                return "<td bgcolor='lightgreen' align='center'><font face='Helvetica,Arial' size='1'>&nbsp;&nbsp;Ok&nbsp;&nbsp;</font></td>";
            case WikiArchive.MODE_UPDATE:
                return "<td bgcolor='lightblue' align='center'><font face='Helvetica,Arial' size='1'>&nbsp;&nbsp;Ok&nbsp;&nbsp;</font></td>";
            default:
                return null;
        }
    } 

/**
 * Outputs a message informing that an action has not been taken for an entry.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm76d8]
 * @return The message string. (Actually used to build a part of an HTML-form user interface.)
 * @param inEntry The entry not processed.
 * @param outEntry The currently existing possible output entry, or null if such does not exist yet.
 * @param mode The action mode, like the <code>mode</code> parameter of performIO().
 */
    protected String msgNotDone(ArchiveEntry inEntry, ArchiveEntry outEntry, int mode) {        
        
        if ((mode & WikiArchive.MODE_SINGLE) != 0) {
            switch (mode & WikiArchive.MODE_BOTH) {
                case WikiArchive.MODE_INSERT:
                    return "<td bgcolor='lightgreen' nowrap align='center' valign='middle'>&nbsp;&nbsp;&nbsp;<input type='checkbox' name='single' value='"+inEntry.getName()+"'/>&nbsp;&nbsp;&nbsp;</td>";
                case WikiArchive.MODE_UPDATE:
                    return "<td bgcolor='lightblue' nowrap align='center' valign='middle'>&nbsp;&nbsp;&nbsp;<input type='checkbox' name='single' value='"+inEntry.getName()+"'/>&nbsp;&nbsp;&nbsp;</td>";
                default:
                    return null;
            }            
        } else {
            return "<td></td>";
        }
    } 

/**
 * Converts Wiki-markup of a bibliography entry page in the Wiki to a ArchiveEntryBibTeX.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm76ca]
 * @return ArchiveEntryBibTeX parsed from the Wiki-page's table entries.
 * @param pagename Name of a Wiki page representing a bibliography entry.
 * @param pagedata The page's Wiki-markup.
 * @param categoryMarker 
 * @see #bib2wiki(ArchiveEntryBibTeX)
 */
    public static ArchiveEntry wiki2entry(String pagename, String pagedata, String categoryMarker) {        
        Map m = parseWikiTableAsMap(pagedata);
        pagedata = WikiToolbox.normalizeLF(pagedata.trim());
        // remove category marker string (usually at the end of the page)
        if (categoryMarker != null) {
            int pos = pagedata.indexOf(categoryMarker);
            if (pos != -1) {
                String s = pagedata.substring(0, pos);
                if ( pagedata.length() > ( pos + categoryMarker.length() ) ) {
                    s += pagedata.substring(pos + categoryMarker.length());
                }
                pagedata = s;
            } else {
                // nop (most likely something wrong, but ignore silently)
            }
        }
        
        String pagetext = getNormalPageText(pagedata);
        pagetext = pagetext.trim();
        
        if (pagetext.length() == 0) {
            pagetext = null;
        }
        ArchiveEntryImpl e = new ArchiveEntryImpl(pagename, m);
        e.setText(pagetext);
        e.setPlainText(pagedata);
        return e;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm76bd]
 * @return 
 * @param entry 
 * @param categoryMarker 
 * @param useOutgoingLinks 
 */
    protected static String entry2wiki(ArchiveEntry entry, String categoryMarker, boolean useOutgoingLinks) {        
        final int WIDTH = 12;
        StringBuffer sb = new StringBuffer();
        
        for (Iterator it = entry.getMap().entrySet().iterator(); it.hasNext(); ) {
            Map.Entry e = (Map.Entry)it.next();
            String key = (String)e.getKey();
            String val = (String)e.getValue();
            if ( (val.length() > 0) || (!entry.isOptional())) {
                key = WikiToolbox.capitalize(key);
                String space = WikiToolbox.repeat( WIDTH - val.length() , ' ' );
                if (key.equals("type")) {
                    val = WikiToolbox.capitalize(val);
                }
                sb.append( "| " + key + space + " | " + val + "\n" );
            }
        }
        
        String pagetext = entry.getText();
        if ( pagetext != null ) {
            sb.append('\n').append(pagetext).append('\n');
        }
        
        if ( ! useOutgoingLinks ) {
            sb.append('\n').append(categoryMarker);
        }
        return sb.toString();
    } 

/**
 * Parses entries of a Wiki table as a key/value-map.
 * The specified string gets scanned for Wiki-table markup (using '|' as marker),
 * each first column of a table is considered to contain key values,
 * each second column is taken as source for an entry-value. Other columns are not used.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm76b6]
 * @return Map wih parsed key/value-pairs.
 * @param s Wiki-text with table markup.
 * @see #bib2wiki(BibEntry)
 */
    protected static Map parseWikiTableAsMap(String s) {        
        Map m = new PropertiesOrdered();
        Collection lines = WikiToolbox.splitLines(s);
        for (Iterator it = lines.iterator(); it.hasNext(); ) {
            String l = (String)it.next();
            if (l.startsWith("|")) {
                Collection row = WikiToolbox.split(l, '|');
                if (row.size()>=2) {
                    Iterator it2 = row.iterator();
                    String key = (String)it2.next();
                    String val = (String)it2.next();
                    if ((key!=null)&&(val!=null)) {
                        m.put(key.trim().toLowerCase(), val.trim());
                    }
                }
            }
        }
        return m;
    } 

/**
 * Returns all normal text in a wiki page, i.e. all text that is not part of a table.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm76ac]
 * @return 
 * @param s 
 */
    protected static String getNormalPageText(String s) {        
        StringBuffer sb = new StringBuffer();
        Collection lines = WikiToolbox.splitLines(s);
        for (Iterator it = lines.iterator(); it.hasNext(); ) {
            String l = (String)it.next();
            if ( ! l.startsWith("|") ) {
                sb.append( l ).append('\n');
            }
        }
        return sb.toString();
    } 
/**
 * Inner class.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7dc0]
 */
public class WikiDataSource {
/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm76a2]
 */
    public WikiEngine engine;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7698]
 */
    public String pagename;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7693]
 */
    public boolean useOutgoingLinks;
 }
 }
