/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.plugin.wikiarchive;

import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.apache.xml.serialize.*;
import org.w3c.dom.*;
import org.xml.sax.InputSource;

/**
 * Archive handler for an XML-represented set of entries.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e4b]
 * @author Jens Gulden
 */
public class ArchiveHandlerXML extends ArchiveHandlerExternalFormatAbstract {

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm768b]
 */
    public static final String CONTENT_TYPE = "text/xml; charset=ISO-8859-1";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7685]
 */
    public static final String CONTENT_SUFFIX = ".xml";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm767f]
 */
    protected static final String TAGNAME_ENTRY = "entry";

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7679]
 */
    protected static final String ATTRIBUTE_ID = "id";

/**
 * Creates an empty data source to be filled for export.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7672]
 * @return 
 */
    public Object createData() throws IOException {        
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder;
        try {
            documentBuilder=dbf.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            throw new IOException("ParserConfigurationException: "+pce.getMessage());
        }
        Document doc = documentBuilder.newDocument();
        Element root = doc.createElement("wiki-archive");
        doc.appendChild(root);
        return doc;
    } 

/**
 * Converts data source to string for export.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm766a]
 * @return 
 * @param data 
 */
    public String dataToString(Object data) throws IOException {        
        StringWriter writer = new StringWriter();
        OutputFormat outputFormat=new OutputFormat();
        outputFormat.setIndenting(true);
        XMLSerializer serializer = new XMLSerializer(writer, outputFormat);
        DOMSerializer domSerializer=serializer.asDOMSerializer();
        domSerializer.serialize((Document)data);
        return writer.toString();
    } 

/**
 * Converts a string to a handled data source for import.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7661]
 * @param s 
 * @param data 
 */
    public void stringToData(String s, Object data) throws IOException {        
        Document doc = (Document)data;
        Element root = doc.getDocumentElement();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder documentBuilder = dbf.newDocumentBuilder();
            InputSource in = new InputSource(new StringReader(s));
            Document inDoc = documentBuilder.parse(in);
            Element inRoot = inDoc.getDocumentElement();
            if ( inRoot != null ) {
                inRoot = (Element)doc.importNode(inRoot, true);
                Node child = inRoot.getFirstChild();
                while ( child != null ) {
                    Node childNext = child.getNextSibling();
                    root.appendChild( child );
                    child = childNext;
                }
            }
        } catch (Exception exc) {
            throw new IOException(exc.getClass().getName() + ": " + exc.getMessage());
        }
    } 

/**
 * Content type for export via HTTP.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm765a]
 * @return 
 */
    public String getContentType() {        
        return CONTENT_TYPE;
    } 

/**
 * Content type for export via HTTP.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7654]
 * @return 
 */
    public String getContentSuffix() {        
        return CONTENT_SUFFIX;
    } 

/**
 * Creates a new archive entry in the data source handled by this ArchiveIOHandler.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm764c]
 * @param data The data source handled by this ArchiveHandler, which is of type Document here.
 * @param entry The entry.
 */
    protected void createEntry(Object data, ArchiveEntry entry) {        
        Document doc = (Document)data;
        Element e = entry2xml(doc, entry);
        doc.getDocumentElement().appendChild(e);
    } 

/**
 * Replaces an archive entry in the data source handled by this ArchiveIOHandler.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7641]
 * @param data The data source handled by this ArchiveHandler, which is of type Document here.
 * @param newEntry The new entry.
 * @param oldEntry The old entry.
 */
    protected void replaceEntry(Object data, ArchiveEntry newEntry, ArchiveEntry oldEntry) throws IOException {        
        Document doc = (Document)data;
        Element oldE = findEntryElement(doc, oldEntry.getName());
        if (oldE != null) {
            Element newE = entry2xml(doc, newEntry);
            removeChildren(oldE);
            oldE.setAttribute( ATTRIBUTE_ID , newEntry.getName());
            moveChildren(newE, oldE);
        } else {
            createEntry(doc, newEntry);
        }
    } 

/**
 * Retrieves all archive entries from the data source.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7636]
 * @return 
 * @param data The data source handled by this ArchiveHandler, which is of type StringBuffer here.
 * @throws IOException if an i/o error occurs
 */
    protected Map parseAll(Object data) throws IOException {        
        Map result = new HashMap(); // TreeMap better ? (see ArchiveHandlerBibTeX)
        Document doc = (Document)data;
        Element root = doc.getDocumentElement();
        NodeList nl = root.getElementsByTagName( TAGNAME_ENTRY );
        for ( int i = 0; i < nl.getLength(); i++ ) {
            Element e = (Element)nl.item(i);
            ArchiveEntry entry = xml2entry(e);
            result.put( entry.getName(), entry );
        }
        return result;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7625]
 * @return 
 * @param doc 
 * @param entry 
 */
    protected static Element entry2xml(Document doc, ArchiveEntry entry) {        
        Element e = doc.createElement( TAGNAME_ENTRY );
        e.setAttribute("id", entry.getName());
        for (Iterator it = entry.getMap().entrySet().iterator(); it.hasNext(); ) {
            Map.Entry me = (Map.Entry)it.next();
            String key = (String)me.getKey();
        }
        String text = entry.getText();
        if (text != null) {
            Text textnode = doc.createTextNode('\n' + text + '\n');
            e.appendChild(textnode);
        }
        return e;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm761b]
 * @param doc 
 * @param e 
 * @param key 
 * @param val 
 */
    private static void entry(Document doc, Element e, String key, String val) {        
        Element child = doc.createElement(key);
        if ( ( val != null ) && ( val.length() > 0 ) ) {
            Text text = doc.createTextNode(val);
            child.appendChild(text);
        }
        e.appendChild(child);
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7614]
 * @return 
 * @param e 
 */
    protected static ArchiveEntry xml2entry(Element e) {        
        String name = e.getAttribute( ATTRIBUTE_ID );
        ArchiveEntryImpl entry = new ArchiveEntryImpl(name);
        Node child = e.getFirstChild();
        while ( (child!=null) && ( ! (child instanceof Element) ) ) {
            child = child.getNextSibling();
        }
        while ( child != null ) {
            Element childE = (Element)child;
            String key = childE.getTagName();
            Node text = childE.getFirstChild();
            String val;
            if ( ( text != null ) && ( text instanceof Text ) ) {
                val = ((Text)text).getData();
            } else {
                val = "";
            }
            entry.put( key, val );
            child = child.getNextSibling();
            while ( (child!=null) && ( ! (child instanceof Element) ) ) {
                child = child.getNextSibling();
            }
        }
        // text nodes
        StringBuffer sb = new StringBuffer();
        child = e.getFirstChild();
        while ( (child!=null) && ( ! (child instanceof Text) ) ) {
            child = child.getNextSibling();
        }
        while ( child != null ) {
            Text textnode = (Text)child;
            String text = textnode.getData();
            sb.append(text);
            child = child.getNextSibling();
            while ( (child!=null) && ( ! (child instanceof Text) ) ) {
                child = child.getNextSibling();
            }
        }
        String text = sb.toString().trim();
        if ( text.length() > 0 ) {
            entry.setText(text);
        }
        return entry;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm760c]
 * @return 
 * @param doc 
 * @param tagname 
 */
    protected static Element findEntryElement(Document doc, String tagname) {        
        NodeList nl = doc.getDocumentElement().getElementsByTagName(tagname);
        if ( nl.getLength() > 0 ) {
            Element e = (Element)nl.item(0);
            return e;
        } else {
            return null;
        }
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7605]
 * @param e 
 */
    protected static void removeChildren(Element e) {        
        Node n = e.getFirstChild();
        while ( n != null ) {
            Node next = n.getNextSibling();
            e.removeChild(n);
            n = next;
        }
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm75fd]
 * @param from 
 * @param to 
 */
    protected static void moveChildren(Element from, Element to) {        
        Node n = from.getFirstChild();
        while ( n != null ) {
            Node next = n.getNextSibling(); // ???
            to.appendChild(n);
            next = n;
        }
    } 
 }
