/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.tags;

import com.ecyrd.jspwiki.WikiContext;
import com.ecyrd.jspwiki.WikiEngine;
import com.ecyrd.jspwiki.WikiPage;
import com.ecyrd.jspwiki.tags.IteratorTag;
import com.ecyrd.jspwiki.tags.WikiTagBase;
import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import de.tuberlin.cs.flp.jspwiki.tags.CategoryIteratorInfo;
import de.tuberlin.cs.flp.jspwiki.util.*;
import java.io.IOException;
import java.util.*;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;

/**
 * <p><strong>&lt;wiki:CategoryIterator [id='...'] [page='...']&gt;<br/>
 * &nbsp;&nbsp;&nbsp; ...<br/>
 * &lt;/wiki:CategoryIterator&gt;</strong></p>
 * <p>Iterates over all categories in which the page set by parameter 'page' (or the current page if 'page' is ommitted) is member. The Java variable in which the category name appears can be set by parameter id.</p>
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e4e]
 * @author Jens Gulden
 */
public class CategoryIteratorTag extends IteratorTag {

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6f96]
 */
    public static final String DEFAULT_ID = "category";

/**
 * The name of the page which is tested whether it refers to the current page.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6f90]
 */
    protected String m_pageName;

/**
 * Sets the page name.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6f89]
 * @param page The page name.
 */
    public void setPage(String page) {        
        m_pageName = page;
    } 

/**
 * Returns the page name.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6f82]
 * @return 
 */
    public String getPage() {        
        return m_pageName;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6f7c]
 * @return 
 */
    public int doStartTag() {        
        m_wikiContext = (WikiContext) pageContext.getAttribute( WikiTagBase.ATTR_CONTEXT, PageContext.REQUEST_SCOPE );
        WikiEngine engine = m_wikiContext.getEngine();
        CategoryManager cm = CategoryManager.getInstance(engine);
        String id = getId(); // never null (id must be known at compile time, so tag-attribute id must be set)
        WikiPage page;
        if( m_pageName != null ) {
            page = engine.getPage( m_pageName );
        } else {
            page = m_wikiContext.getPage(); // will _not_ return null if page doesn't exist yet
        }
        if( (page != null) ) {
        	String n = page.getName();
            Collection cats = cm.getCategories( n );
            if( cats != null ) {
                m_iterator = cats.iterator();
                if( m_iterator.hasNext() ) {
                    Object o = m_iterator.next();
                    String cat = (String)o;
                    // Inside the iterator's body, the current page will be the category page referred to.
                    // This resembles the behaviour of e.g. AttachmentIteratorTag.
                    WikiContext context = (WikiContext)m_wikiContext.clone();
                    context.setPage( engine.getPage( cat ) );
                    pageContext.setAttribute( WikiTagBase.ATTR_CONTEXT, context, PageContext.REQUEST_SCOPE );
                    pageContext.setAttribute( id, cat );                   
                    return EVAL_BODY_TAG;
                } else {
                    return SKIP_BODY;
                }
            } else {
                //log.debug("No attachments to display.");
                // There are no attachments included
                return SKIP_BODY;
            }
        } else {
            return SKIP_BODY;
        }
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6f76]
 * @return 
 */
    public final int doAfterBody() {        
        if( bodyContent != null )
        {
            try
            {
                JspWriter out = getPreviousOut();
                out.print(bodyContent.getString());
                bodyContent.clearBody();
            }
            catch( IOException e )
            {
                //log.error("Unable to get inner tag text", e);
                // FIXME: throw something?
            }
        }
        if( m_iterator != null && m_iterator.hasNext() )
        {
            Object o = m_iterator.next();
            String category = (String) o;
            WikiContext context = (WikiContext)m_wikiContext.clone();
            WikiPage page = context.getEngine().getPage(category);
            context.setPage( page );
            pageContext.setAttribute( WikiTagBase.ATTR_CONTEXT,
                                      context,
                                      PageContext.REQUEST_SCOPE );
            return EVAL_BODY_TAG;
        }
        else
        {
            return SKIP_BODY;
        }
    } 
 }
