/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.tags;

import com.ecyrd.jspwiki.*;
import com.ecyrd.jspwiki.attachment.*;
import com.ecyrd.jspwiki.providers.ProviderException;
import com.ecyrd.jspwiki.tags.WikiTagBase;
import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import de.tuberlin.cs.flp.jspwiki.util.*;
import java.io.IOException;
import java.util.*;
import javax.servlet.jsp.JspWriter;

/**
 * <p><strong>&lt;wiki:FileIcon filename='...'/&gt;</strong></p>
 * <p>Inserts a file icon registered as attachment on page FileIcons (see jspwiki.fileicons.page option), suitable for the filename specified. For example, filename=&quot;test.pdf&quot; would insert the typical pdf-logo icon, supposed this filetype has been properly configured on the FileIcons page (or as configured in jspwiki.properties).</p>
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e41]
 * @author Jens Gulden
 */
public class FileIconTag extends WikiTagBase {

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6fd7]
 */
    private String m_filename = null;

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6fd0]
 * @return 
 */
    public String getFilename() {        
        return m_filename;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6fc9]
 * @param filename 
 */
    public void setFilename(String filename) {        
        m_filename = filename;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6fc3]
 * @return 
 */
    public final int doWikiStartTag() throws IOException {        
        WikiEngine engine = m_wikiContext.getEngine();
        JspWriter out = pageContext.getOut();
        String filename = getFilename();
        String icon = getFileIcon(engine, filename);
        out.print("<img src=\""+icon+"\" border=\"0\">");
        return SKIP_BODY;
    } 

/**
 * Also called by WikiArchive.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6fba]
 * @return 
 * @param engine 
 * @param filename 
 */
    public static String getFileIcon(WikiEngine engine, String filename) {        
        String fileiconsPage = engine.getWikiProperties().getProperty("jspwiki.fileicons.page", "FileIcons"); // list of available icons, stored as attachments
        WikiPage page = engine.getPage(fileiconsPage);
        if (page == null) {
            return "FileIconsPageNotFound";
        }
        String suffix;
        int pos = filename.lastIndexOf('.');
        if (pos-1 < filename.length()) {
            suffix = filename.substring(pos+1);
        } else {
            suffix = "default";
        }
        AttachmentManager am = engine.getAttachmentManager();
        try {
            Collection attachments = am.listAttachments(page);
            for (Iterator at = attachments.iterator(); at.hasNext(); ) {
                Attachment a = (Attachment)at.next();
                String attachmentFilename = a.getFileName();
                pos = attachmentFilename.lastIndexOf('.');
                String icontype = attachmentFilename.substring(0, pos);
                if (icontype.equals(suffix)) {
                    return fileiconsPage+"/"+attachmentFilename; // successful return
                }
            }
        } catch (ProviderException pe) {
            //fallthrough
        }
        // not found:
        return "FileIconsPageNotFound";
    } 
 }
