/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.tags;

import com.ecyrd.jspwiki.WikiEngine;
import com.ecyrd.jspwiki.WikiPage;
import com.ecyrd.jspwiki.tags.WikiTagBase;
import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import de.tuberlin.cs.flp.jspwiki.util.*;

/**
 * <p><strong>&lt;wiki:IsCategory [page='...']&gt;<br/>
 * &nbsp;&nbsp;&nbsp; ...<br/>
 * &lt;/wiki:IsCategory&gt;</strong></p>
 * <p>Evaluates the tag-body if the page is a category page. Default for parameter page is this page.</p>
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e48]
 * @author Jens Gulden
 */
public class IsCategoryTag extends WikiTagBase {

/**
 * The name of the page which is tested whether it refers to the current page.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6fb2]
 */
    protected String m_pageName;

/**
 * Sets the page name.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6fab]
 * @param page The page name.
 */
    public void setPage(String page) {        
        m_pageName = page;
    } 

/**
 * Returns the page name.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6fa4]
 * @return 
 */
    public String getPage() {        
        return m_pageName;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6f9e]
 * @return 
 */
    public int doWikiStartTag() {        
        WikiEngine engine = m_wikiContext.getEngine();
        CategoryManager cm = CategoryManager.getInstance(engine);
                
        WikiPage page;
        if( m_pageName != null ) {
            page = engine.getPage( m_pageName );
        } else {
            page = m_wikiContext.getPage();
        }
        
        if (page != null) {
            if ( cm.isCategory(page.getName()) ) {
                return EVAL_BODY_INCLUDE;
            } else {
                return SKIP_BODY;
            }
        } else {
            return SKIP_BODY;
        }
    } 
 }
