/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.tags;

import com.ecyrd.jspwiki.WikiEngine;
import com.ecyrd.jspwiki.WikiPage;
import com.ecyrd.jspwiki.tags.WikiTagBase;
import de.tuberlin.cs.flp.jspwiki.CategoryManager;
import de.tuberlin.cs.flp.jspwiki.util.*;
import java.io.IOException;
import java.util.*;

/**
 * <p><strong>&lt;wiki:PageReferredBy page='...'&gt;<br/>
 * &nbsp;&nbsp;&nbsp; ...<br/>
 * &lt;/wiki:PageReferredBy&gt;</strong></p>
 * <p>Implements a JSP-tag named &quot;&lt;wiki:PageReferredBy&gt;&quot; which includes its body into the page if this page is referred from the page given by parameter page. (I.e., if 'page' contains a link to this page.)</p>
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e43]
 * @author Jens Gulden
 */
public class PageReferredByTag extends WikiTagBase {

/**
 * The name of the page which is tested whether it refers to the current page.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6ff7]
 */
    protected String m_pageName;

/**
 * Sets the page name.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6ff0]
 * @param page The page name.
 */
    public void setPage(String page) {        
        m_pageName = page;
    } 

/**
 * Returns the page name.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6fe9]
 * @return 
 */
    public String getPage() {        
        return m_pageName;
    } 

/**
 * Decides whether to include the body part or not.
 * The body part is included if the page named by <code>getPage()</code>
 * includes a link to the current page.
 * Known bug: if this page's name contains special chars (e.g. German umlauts)
 * an existing reference from the referring page may not be found.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6fe3]
 * @return evaluation result code
 * @throws IOException if an i/o error occurs
 */
    public final int doWikiStartTag() throws IOException {        
        WikiEngine engine = m_wikiContext.getEngine();
        WikiPage thisPage = m_wikiContext.getPage();
        
        String referringPage = this.getPage();
        Collection referrers = engine.getReferenceManager().findReferrers( thisPage.getName() );
        if ( (thisPage != null) && (referrers != null) && ( referrers.contains( referringPage ) ) ) {
            return EVAL_BODY_INCLUDE;
        } else {
            return SKIP_BODY;
        }
    } 
 }
