/*
    WikiArchive - Category and Archive extensions for JSPWiki
    
    Copyright (C) 2007 Jens Gulden (jgulden@cs.tu-berlin.de)
    JSPWiki copyright (c) Janne Jalkanen (Janne.Jalkanen@iki.fi)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.tuberlin.cs.flp.jspwiki.util;

import java.io.*;
import java.util.*;

/**
 * Class PropertiesOrdered. This behaves like java.util.Properties, but preserves the order in which entries have been loaded or added. This is useful if the order of configuration entries shold be preserved when Properties are loaded.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7e3f]
 * @author Jens Gulden
 */
public class PropertiesOrdered extends Properties {
/**
 * The keys in order.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6f1c]
 */
    protected PropertiesOrdered.ArraySet keysInOrder = new ArraySet();

/**
 * Creates a new instance of PropertiesOrdered
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6f14]
 */
    public  PropertiesOrdered() {        
        super();
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6f0e]
 * @return 
 */
    public Set entrySet() {        
        ArraySet r = new ArraySet();
        for (Iterator it = keysInOrder.iterator(); it.hasNext(); ) {
            Object key = it.next();
            Object val = this.get(key);
            r.add(new MapEntryImpl(key, val));
        }
        return r;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6f08]
 * @return 
 */
    public Set keySet() {        
        return Collections.unmodifiableSet(keysInOrder);
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6eff]
 * @return 
 */
    public Enumeration keys() {        
        return Collections.enumeration(keySet());
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6ef9]
 * @return 
 */
    public Collection values() {        
        ArraySet r = new ArraySet();
        for (Iterator it = keysInOrder.iterator(); it.hasNext(); ) {
            Object key = it.next();
            Object val = this.get(key);
            r.add(val);
        }
        return r;
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6ef3]
 * @return 
 */
    public Enumeration elements() {        
        return Collections.enumeration(values());
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6eeb]
 * @return 
 * @param key The key.
 * @param val The val.
 */
    public Object put(Object key, Object val) {        
        if (!this.keysInOrder.contains(key)) {
            this.keysInOrder.add(key);
        } else {
            super.remove(key);
        }
        return super.put(key, val);
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6ee2]
 * @return 
 * @param key The key.
 */
    public Object remove(Object key) {        
        this.keysInOrder.remove(key);
        return super.remove(key);
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6eda]
 * @param in The input stream.
 * @throws IOException if an i/o error occurs
 */
    public void load(InputStream in) throws IOException {        
        BufferedReader buf = new BufferedReader(new InputStreamReader(in));
        keysInOrder = new ArraySet();
        StringBuffer sb = new StringBuffer();
        String line;
        while ((line = buf.readLine()) != null) {
            if (!line.startsWith("#")) {
                int pos = line.indexOf('=');
                if (pos!=-1) {
                    String key = line.substring(0, pos);
                    String val = line.substring(pos+1);
                    key = key.trim();
                    this.put(key, val);
                }
            }
        }
        in.close();
    } 
/**
 * Inner class ArraySet.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7c59]
 * @author Jens Gulden
 */
private class ArraySet extends ArrayList implements Set {

/**
 * The entries.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6ece]
 */
    private ArrayList entries = new ArrayList();

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6ec6]
 * @return 
 * @param o The object.
 */
    public boolean add(Object o) {        
            return entries.add(o);
    } 

/**
 * Adds a all.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6ebe]
 * @return 
 * @param c The collection.
 */
    public boolean addAll(Collection c) {        
            return entries.addAll(c);
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6eb6]
 * @return 
 * @param o The object.
 */
    public boolean contains(Object o) {        
            return entries.contains(o);
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6eae]
 * @return 
 * @param c The collection.
 */
    public boolean containsAll(Collection c) {        
            return entries.containsAll(c);
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6ea7]
 * @return 
 */
    public Iterator iterator() {        
            return entries.iterator();
    } 

/**
 * Removes a all.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6ea0]
 * @return 
 * @param c The collection.
 */
    public boolean removeAll(Collection c) {        
            return entries.removeAll(c);
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6e98]
 * @return 
 * @param c The collection.
 */
    public boolean retainAll(Collection c) {        
            return entries.retainAll(c);
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6e90]
 * @return 
 */
    public Object[] toArray() {        
            return entries.toArray();
    } 

/**
 * 
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6e89]
 * @return 
 * @param a The object array.
 */
    public Object[] toArray(Object[] a) {        
            return entries.toArray(a);
    } 
 }
/**
 * Inner class MapEntryImpl.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm7c57]
 * @author Jens Gulden
 */
private class MapEntryImpl implements java.util.Map.Entry {

/**
 * The key.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6e7f]
 */
    private Object key;

/**
 * The val.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6e7a]
 */
    private Object val;

/**
 * Creates a new instance of MapEntryImpl.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6e72]
 * @param key The key.
 * @param val The val.
 */
    private  MapEntryImpl(Object key, Object val) {        
        this.key = key;
        this.val = val;
    } 

/**
 * Returns the key.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6e6a]
 * @return 
 */
    public Object getKey() {        
        return key;
    } 

/**
 * Returns the value.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6e64]
 * @return 
 */
    public Object getValue() {        
        return val;
    } 

/**
 * Sets the value.
 * 
 * @poseidon-object-id [Im7d594a32m112753776a4mm6e5d]
 * @return 
 * @param value The value.
 */
    public Object setValue(Object value) {        
        this.val = value;
        return value;
    } 
 }

    /** @poseidon-generated */
    public PropertiesOrdered.ArraySet getKeysInOrder() {
        return keysInOrder;
    }
    /** @poseidon-generated */
    public void setKeysInOrder(PropertiesOrdered.ArraySet arraySet) {
        this.keysInOrder = arraySet;
    }
 }
